######## Processing of Illumina microarray data #########
setwd("C:/Users/GianniMonaco/Desktop/Work/Deconvolution/Microarrays")

library(lumi)

lumiobj <- lumiR("E0162-16 AL Gianni Raw.txt", annotationColumn=c('ILMN GENE', 'SYMBOL'))
symb <- featureData(lumiobj)@data

### pairs correlations
png(filename = "pairs_raw.png", width = 3000, height = 3000)
pairs(lumiobj)
dev.off()

### hierarchical cluster
png(filename = "cluster_raw.png", width = 400, height = 400)
plotSampleRelation(lumiobj) # the same as: plot(lumiobj, what='sampleRelation')
dev.off()

### boxplot
png(filename = "boxplot_raw.png", width = 600, height = 400)
boxplot(lumiobj) # the same as: plot(lumiobj, what='sampleRelation')
dev.off()

lumiobjT <- lumiT(lumiobj, method="log2")

lumiobjTN <- lumiN(lumiobjT, method="quantile")

## QC plots
png(filename = "pairs_TN.png", width = 3000, height = 3000)
pairs(lumiobjTN)
dev.off()
png(filename = "cluster_TN.png", width = 400, height = 400)
plotSampleRelation(lumiobjTN) # the same as: plot(lumiobj, what='sampleRelation')
dev.off()
png(filename = "boxplot_TN.png", width = 600, height = 400)
boxplot(lumiobjTN) # the same as: plot(lumiobj, what='sampleRelation')
dev.off()


## venn diagram
expr <- exprs(lumiobjTN)
colnames(expr)
foldCh_34SP <- expr[,1] - expr[,7] 
foldCh_3H37 <- expr[,2] - expr[,8]
foldCh_CR3L <- expr[,3] - expr[,9]

library(limma)
diff1 <- foldCh_34SP > 2
diff2 <- foldCh_3H37 > 2
diff3 <- foldCh_CR3L > 2
c3 <- cbind(diff1, diff2, diff3)
a <- vennCounts(c3)
png(filename = "vennDiff.png", width = 400, height = 400)
vennDiagram(a, names = c("34SP", "3H37", "CR3L"), circle.col = c("red","blue", "green"))
dev.off()

diff1 <- which(foldCh_34SP > 2)
diff2 <- which(foldCh_3H37 > 2)
diff3 <- which(foldCh_CR3L > 2)
diffUN <- unique(unique(names(diff1), names(diff2)), names(diff3))
diffUN_sym <- symb[diffUN,2]
write.table(diffUN_sym,"diffUN.txt",quote=F, row.names=F,col.names=F)


### Calculate the scatterplot after you remove the filtered genes
lumiobjTN2 <- lumiobjTN[-which(rownames(lumiobjTN) %in% diffUN), ]
plotSampleRelation(lumiobjTN2)
png(filename = "pairs_TN2.png", width = 3000, height = 3000)
pairs(lumiobjTN2)
dev.off()
