options(stringsAsFactors = FALSE)
# load the libraries
libraries <- "tidyverse,RColorBrewer,forestplot"
libraries <- strsplit(libraries, ",")[[1]]
for (name in libraries) {
  suppressPackageStartupMessages(library(name, character.only=T, warn.conflicts=F, quietly=T, verbose=F))
}
SaveName <- "kal_genes" # assign a value between STAR, kal_tx and kal_genes
source("~/Dropbox/_Deconvolution_paper/Src/4.RNAseqObj.R")

# Load differentially expressed genes
namenorm <- dir(pattern="tpm")[6]
DEtot <- read.table(paste0(namenorm, "/DEtot.txt"), sep="\t", header = T, check.names = FALSE)
dir.create(paste0(namenorm,"/enrichment"), showWarnings = FALSE)

# #prepare Reactome Data
# React <- read.delim("../enrichment/Ensembl2Reactome_All_Levels.txt", comment.char = "", stringsAsFactors = F, sep = "\t",header = FALSE)
# ReactRel <- read_tsv("../enrichment/ReactomePathwaysRelation.txt", comment = "", col_names = F) # with readr
# ReactALLHUM <- React %>% filter(V6 == "Homo sapiens") %>% select(V1,V2,V4) %>% distinct()   # to remove the rows that are duplicated where the same gene has been found manually and electronically
# ReactALL <- ReactALLHUM %>% select(V1,V2,V4) %>% nest(V1) %>% mutate(data = map(data, ~unlist(.x))) %>% mutate(len=map_int(data, ~length(.x)))   # here you have list columns
# universe <- unique(ReactALLHUM$V1)
# save(file = "../enrichment/ReactUniverse.RData", universe)
# # order them
# ReactRel <- ReactRel[grep("HSA", ReactRel$X1),]
# rootPath <- unique(ReactRel[which(!ReactRel$X1 %in% ReactRel$X2),]$X1) # select all parent patheways
# rootPath <- reactALL %>% filter(V2 %in% rootPath) %>% arrange(V4) %>% select(V2) %>% unlist() # order them by alphabetic order
# # find hierarchy 
# ReactOrd <- c()
# ReactRel2 <- ReactRel
# for(l1 in rootPath){
#   lev<-1
#   ReactOrd <- rbind( ReactOrd, c( l1, lev, filter(reactALL, V2 == l1)$V4))  
#   parent <-l1
#   while(parent %in% ReactRel2$X1){
#     childs <- ReactRel2 %>% filter(X1 %in% parent) %>% select(X2) %>% unlist()  #find childs
#     if(all(childs %in% reactALL$V2))  childs <- reactALL %>% filter(V2 %in% childs) %>% arrange(V4) %>% select(V2) %>% unlist() # order them alphabetically
#     child <-childs[1]  #select only the first child
#     if(child %in% ReactALL$V2) ReactOrd <- rbind( ReactOrd, c( child, lev+1, filter(ReactALL, V2 == child)$V4)) 
#     ReactRel2 <- ReactRel2 %>% filter(X1!=parent | X2 !=child)   # remove the row
#     if(child %in% ReactRel2$X1){
#       lev <- lev+1
#       parent <- child
#     }else if(parent %in% ReactRel2$X1){
#       # do nothing
#     }else{
#       while(!parent %in% ReactRel2$X1 & parent != l1){
#         lev <- lev-1 
#         idLev <- which(ReactOrd[,2] == lev) # all rows with a previous parent level
#         parent <- ReactOrd[idLev[length(idLev)], 1] # take the last one recorded 
#         if(length(parent)==0) parent= l1
#       }
#     }
#   }
# }
# 
# ReactOrd2 <- cbind(ReactOrd, reactALL[match(ReactOrd[,1], ReactALL$V2),c(3,4)], 1:nrow(ReactOrd))
# colnames(ReactOrd2) <- c("id", "level", "name","genes", "length", "numbering")
# #remove duplicates in the lower levels
# ReactOrd3 <- ReactOrd2[order(ReactOrd2[,2]),]
# ReactOrd4 <- ReactOrd3[!duplicated(ReactOrd3[,1]), ]
# ReactOrd5 <- ReactOrd4[order(ReactOrd4$numbering), ]
# # ReactOrd5 <- ReactOrd5 %>% filter(len > 3)  # select only gene sets bigger than 3
# ReactOrdered <- ReactOrd5
# save(file = "../enrichment/Reactordered.RData", ReactOrdered)

load("../enrichment/Reactordered.RData")
load("../enrichment/ReactUniverse.RData")


DEtot[,1] <- substr(DEtot[,1],1,15) # remove the version number from ensemble IDs
DEtot2 <- DEtot[which(DEtot[,1] %in% universe),] #remove rows that are not contained in the universe
DEtot2 <- DEtot2[!duplicated(DEtot2[,1]),]  
universe <- universe[universe %in% DEtot2[,1]]


res <- c()
for(i in 1:nrow(ReactOrdered)){
  vect <- ReactOrdered[i,]$genes[[1]]  
  vect <- vect[vect %in% DEtot2[,1]]
  
  temp <-c()
  for(ct in colnames(DEtot)[4:ncol(DEtot)]){
    de <- na.omit(unique(DEtot2[which(DEtot2[,ct] != 0), 1]))
    notde <- DEtot2[!DEtot2[,1] %in% de, 1]
    
    GSde <- sum(vect %in% de) # differentially expressed in the Gene set
    GSnotde <- sum(vect %in% notde)
    RESTde <- length(de) - GSde
    RESTnotde <- length(notde) - GSnotde
    contigency.matrix <- rbind( c(GSde, GSnotde), c(RESTde, RESTnotde) )
    pval <- signif(fisher.test( contigency.matrix, alternative="greater")$p.value, 3) 
    # phyper(GSde-1, length(vect), length(universe)-length(vect), length(de), lower.tail=FALSE) #it gives the same results
    if(pval <0.05) temp <- c(temp, paste0( ct, " (", pval,")"))  # ifelse(!is.null(temp), ", ", ""),
  }
  if(is.null(temp)){ temp <- "" }else{ temp<- paste(temp, collapse=", ") }
  res <- rbind(res, c(unlist(ReactOrdered[i,c(1,2,3,5)]), length(vect),temp))
  print(i)
}
colnames(res) <- c("Reactome ID", "level", "Name Pathway", "size", "size tested", "cell type enriched (p-value)")

write.table(res, paste0(namenorm, "/enrichment/REACTOMEenrich.txt"), sep = "\t", col.names=NA)


