libraries <- strsplit("ggplot2,circlize,rtracklayer,RColorBrewer,ComplexHeatmap", ",")[[1]]
for (name in libraries) {
  suppressPackageStartupMessages(library(name, character.only=T, warn.conflicts=F, quietly=T, verbose=F))
}

SaveName <- "kal_genes" # assign a value between STAR, kal_tx and kal_genes
source("~/Dropbox/_Deconvolution_paper/Src/4.RNAseqObj.R") 
dir.create("circos", showWarnings = FALSE)

namenorm <- dir(pattern="tpm")[1]
DEtot <- read.table(paste0(namenorm, "/DEtot.txt"), sep="\t", header = T, check.names = FALSE) #log FC of genes with FDR <0.05. the remaining values are 0

### gene ranges
gencode <- import("../genes_info/gencode.v26.annotation.ercc.gtf", colnames=c("gene_id","gene_name","gene_type"), feature.type="gene") # find the feature type in the third column of the gtf file
bedtemp <- as.data.frame(gencode)[1:3]
rownames(bedtemp) <- gencode@elementMetadata@listData$gene_id


### filter out genes you don't want
counts2 <- counts[!duplicated(counts),]  # it removes genes that have always 0 expression and the ones with same expression (derived from some weird kallisto calculation)
counts3 <- counts2[grepl("^ENS", rownames(counts2)),] # removes ERCC sequences
counts4 <- counts3[as.character(bedtemp[rownames(counts3),1]) != "chrM", ]

# calculate tpm
tpm <- tpm.standard(counts4, len[rownames(counts4), ])
# calculate tpm percentage of each gene
tpmPer <- (tpm/10e+6) *100
colnames(tpmPer) <- pheno$Sample_Name

# subset bed file
bed <- bedtemp[rownames(tpm), ]

# read cytoband and create segments to color for each chromosome
cytoband <- read.table("../genes_info/cytoBand.txt")
cyt2 <- cytoband[cytoband[,5] != "acen",]  # remove centromere
SEGM<-c()  # it will store the segments
chr <- c()
for(i in unique(cyt2$V1)){
  sub <- subset(cyt2, V1 == i)
  for(side in c("p", "q")){
    reg <- sub[grepl(side,sub[,4]), c(2,3)]
    parts <- round((max(reg) - min(reg)) / 1.5e+07)  # define here the size of each chromosome bin you intend to get
    seg<-(max(reg) - min(reg))/parts
    if(parts == 1) SEGM <- rbind(SEGM, c( min(reg), max(reg)))
    if(parts == 2) {
      SEGM <- rbind(SEGM, c( min(reg), min(reg)+ seg))
      SEGM <- rbind(SEGM, c( min(reg)+ seg, max(reg)))
    }
    if(parts >= 3) {
      SEGM <- rbind(SEGM, c(min(reg), min(reg)+ seg))
      for(a in 2:(parts-1)){
        SEGM <- rbind(SEGM, c(SEGM[nrow(SEGM),2], SEGM[nrow(SEGM),2] + seg))
      }
      SEGM <- rbind(SEGM, c(SEGM[nrow(SEGM),2], max(reg)))
    }
    chr <-c(chr, rep(i, parts))
  }
}
# combine chromosome and segments
SEGM2 <- data.frame(chr=chr, start=SEGM[,1], end=SEGM[,2], stringsAsFactors = F)

# generate expression value for each segment
bedtpm<-c()
for(i in 1:nrow(SEGM2)){  #find gene expression for each band
 subbed <- subset(bed, seqnames==SEGM2[i,"chr"] & end>= SEGM2[i,"start"] & end<= SEGM2[i,"end"]) 
 if(nrow(subbed) == 0 ) bedtpm <- rbind(bedtpm, rep(0, ncol(tpm)))
 if(nrow(subbed) == 1 ) bedtpm <- rbind(bedtpm, tpm[rownames(subbed),])
 if(nrow(subbed) > 1 ) bedtpm <- rbind(bedtpm, colSums(tpm[rownames(subbed),]))
}
# find tpm percentage
bedPer <- (bedtpm/10e+6) *100
bedPer <- round(bedPer,digits = 3) # round the percentage
max(bedPer)  # check it! It should be around 4, otherwise change the color function and legend

# all the cell types
celltypes <- unique(as.character(pheno$group))

# create color ramp
br <- c(seq(0,0.25,0.25/5),0.5,1,2, 4)
#f = colorRamp2(breaks = br, colors = c("white",brewer.pal(9, "YlOrBr")[2:9], "black"))
f = colorRamp2(breaks = br, colors = c("white",brewer.pal(7, "YlOrRd"), brewer.pal(9, "YlOrBr")[9],"black")) #colors taken from YlOrRd and YlOrBr

# create the legend!
pdf(file = "circos/legend.pdf",width = 2, height = 2.1)
#lgd_expr <- Legend(at = c("class1", "class2"), title = "points", type = "points", legend_gp = gpar(col = 2:3))
lgd_expr <- Legend(at = br, legend_gp = gpar(col =f(br)),type="points",grid_height=unit(4, "mm"), pch = 15, size =unit(5, "mm"),background = "white",
                  labels_gp = gpar(fontsize = 8),title_position = "topleft", title = "percentage of expression",border="grey",
                  title_gp = gpar(fontsize = 10, fontface = "bold")) # it creates a grob object
pushViewport(viewport())
# x = unit(2, "mm"), y = unit(4, "mm"),  width = grobWidth(lgd_expr), height=grobHeight(lgd_expr), just = c("left", "bottom") # use these parameters if you want to add it in a side of the page
grid.draw(lgd_expr)
upViewport()
dev.off()

fullGENES <- c()
### CREATE CIRCOS PLOT ###
for(celln in 1:length(celltypes)){
  
  # subset of gene expression 
  idx <- as.character(pheno$group) == celltypes[celln]
  bedtemp <- bedPer[,idx] 
  
  if(sum(idx) <= 4){ th <- 0.07   # define the track height for the plots
  }else{ th <- 0.01 }
  #find the genes to report
  tpmtemp <- tpmPer[,idx] 
  genes <- rownames(tpmtemp)[apply(tpmtemp, 1, function(x) any(x>0.05))] # that have a proportion higher than 0.1
  bedSUB<- bed[genes,]
  geneNames <- features[genes,3]
  fullGENES <- unique(c(fullGENES, geneNames))
  
  # mark differentially expressed genes
  if(celltypes[celln] != "PBMCs"){
  DEgenesP <- as.character(DEtot[ DEtot[,celltypes[celln]] > 0  ,  "gene_name"])
  #DEgenesN <- as.character(DEtot[ DEtot[,celltypes[celln]] < 0  ,  "gene_name"])
  geneNames[geneNames %in% DEgenesP ] <- paste0(geneNames[geneNames %in% DEgenesP ], "*")
  #geneNames[geneNames %in% DEgenesN ] <- paste0(geneNames[geneNames %in% DEgenesN ], "-")
  }
  
  # add the gene names as the fourth column
  bedSUB[,4] <- geneNames
  
  pdf(file = paste0("circos/", sub("/","-",celltypes[celln]),".pdf") ,width =5, height = 4.9)
  par(mar = c(1, 1, 1, 1))
  # inizialize circos plot
  circos.initializeWithIdeogram(species = "hg19",plotType =c("ideogram","labels")) # it plots the cytoband of the hg19 genome
  om = circos.par("track.margin") #save margin sizes
  oc = circos.par("cell.padding")
  # add heatmap tracks
  circos.par(track.margin = c(om[1], 0.01), cell.padding = c(0, 0, 0, 0))
  for(col in 1:ncol(tpmtemp)){
    SEGM3 <- cbind(SEGM2, bedtemp[,col])
    if(grepl("9JD4|G4YW|CR3L|684C", colnames(tpmtemp)[col])) SEGM3 <- SEGM3[SEGM3$chr != "chrY", ]  # remove Y chromosome from female donors
    circos.genomicTrackPlotRegion(SEGM3, stack =F, panel.fun = function(region, value, ...) {
      circos.genomicRect(region, value, col = f(value[[1]]),
                         border = "grey", lwd = 0.1, posTransform = posTransform.default, ...)
    }, bg.border = NA, track.height = th)
  }
  #circos.genomicPosTransformLines(bedSUB, posTransform = posTransform.default, horizontalLine = "top")
  circos.genomicLabels(bedSUB, labels.column = 4, side= "inside", cex=0.7,connection_height=0.04)
  circos.par(track.margin = om, cell.padding = oc)
  title(celltypes[celln])
  circos.clear()
  dev.off()
}

# HERE CHECK THE GENES THAT ARE EXPRESSED IN BROADER GROUPS
write.table(DEtot[DEtot[, "gene_name"] %in% fullGENES,], "circos/selectedGenes.txt", sep = "\t",row.names=F)
