#' # This script aims to put together the flowcytometry proportions
setwd("~/Desktop/Work_temp/Deconvolution/")
library(edgeR)
# Sys.setlocale("LC_ALL", "en_US.ISO8859-1")
#' reading the files
#' 
mono <- read.table("mono_flow.txt", sep="\t", header=T,fill = TRUE, row.names=1, check.names = FALSE)
B <- read.table("B_flow.txt", sep="\t", header=T,fill = TRUE, row.names=1, check.names = FALSE)
B2 <- read.table("B2_flow.txt", sep="\t", header=T,fill = TRUE, row.names=1, check.names = FALSE) # influx
CD4 <- read.table("CD4_flow.txt", sep="\t", header=T,fill = TRUE, row.names=1, check.names = FALSE)
CD8 <- read.table("CD8_flow.txt", sep="\t", header=T,fill = TRUE, row.names=1, check.names = FALSE)

mono <- mono[!rownames(mono) %in% c("Mean","SD"),]
B <- B[!rownames(B) %in% c("Mean","SD"),]
B2 <- B2[!rownames(B2) %in% c("Mean","SD"),]
CD4 <- CD4[!rownames(CD4) %in% c("Mean","SD"),]
CD8 <- CD8[!rownames(CD8) %in% c("Mean","SD"),]

# Change all rownames
rownames(mono) <- mono$SampleID
rownames(B) <- B$SampleID
rownames(B2) <- B2$SampleID
rownames(CD4) <- CD4$SampleID
rownames(CD8) <- CD8$SampleID


#remove unnecessary columns
CD4$SampleID <- NULL
CD8$SampleID <-NULL
mono$SampleID <- NULL
B$SampleID <-NULL
B2$SampleID <-NULL

# make sure they are all ordered
B <- B[rownames(mono),]
CD4 <- CD4[rownames(mono),]
CD8 <- CD8[rownames(mono),]

# Let's start by adjusting the values of the B cell panel for 925L
# calculate effective CD45 numbers from the monocytes of the influx panel:
monoInfl <- B2["925L","CD45minusCD3"] * (B2["925L","Mono"] / 100) # number of monocytes in the influx B panel
monoSym <- mono["925L","CD45"] -  ( mono["925L","CD45"] * (mono["925L", "CD3-CD19"] / 100) )
CD45eff <- (mono["925L","CD45"] / monoSym ) * monoInfl 
colnames(B2)
B["925L","B cells"] <- (B2["925L","CD45minusCD3"] * B2["925L","B cells"]) / CD45eff   # from the symphony is 17.4
B["925L","Progenitors"] <- (B2["925L","CD45minusCD3"] * B2["925L","Progenitors"]) / CD45eff  # from the symphony is 0.41
# now the percentages of B cells are calculated according to the monocytes percentage of the influx panel. In case you think you can trust the percentage of total B cells and progenitors extracted from Symphony, comment the lines above and calculate the CD45eff in this way:
# CD45eff <- (B2["925L","CD45minusCD3"] * B2["925L","B cells"]) * B["925L","B cells"]
# Now make the other calculation are in the form: 
B["925L","B Naive"]<- (B2["925L","CD45minusCD3"] * B2["925L","B Naive"]) / CD45eff
B["925L","B SM"]<- (B2["925L","CD45minusCD3"] * B2["925L","B SM"]) / CD45eff
B["925L","B NSM"]<- (B2["925L","CD45minusCD3"] * B2["925L","B NSM"]) / CD45eff
B["925L","B Ex"]<- (B2["925L","CD45minusCD3"] * B2["925L","B Ex"]) / CD45eff
B["925L","Plasmablasts"]<- (B2["925L","CD45minusCD3"] * B2["925L","Plasmablasts"]) / CD45eff


########## Fix DZQV  #########
# Monocytes first
CD319NeutroOLD <- mono["DZQV","CD3-CD19"] + mono["DZQV","Neutrophils LD"]
mono["DZQV","CD3-CD19"] <- CD8["DZQV","CD3_freq"] + B["DZQV","B cells"]
otherMono <- c("Monocytes C", "Monocytes I", "Monocytes NC",   "NK", "pDCs", "Basophils LD", "mDCs")
TOTOthMonoOLD <- sum(mono["DZQV",otherMono ])
TOTOthMono <- TOTOthMonoOLD-  (mono["DZQV","CD3-CD19"] - CD319NeutroOLD)
mono["DZQV",otherMono ] <-  (mono["DZQV",otherMono ]/TOTOthMonoOLD) *TOTOthMono
# Now B cells
proBcells <- B2["DZQV","B cells"] / B["DZQV","B cells"]
B["DZQV","B Naive"]<- B2["DZQV","B Naive"] / proBcells
B["DZQV","B SM"]<- B2["DZQV","B SM"] / proBcells
B["DZQV","B NSM"]<- B2["DZQV","B NSM"] / proBcells
B["DZQV","B Ex"]<-  B2["DZQV","B Ex"] / proBcells
B["DZQV","Plasmablasts"]<- B2["DZQV","Plasmablasts"] / proBcells


# Remove some unnecessary columns
CD8$CD3_freq <- NULL
CD4$CD3_freq <- NULL
CD4$CD3 <- NULL
CD8$CD3 <-NULL
CD8$CD4 <-NULL

# bind CD4 and CD8 panels and find the percentage of CD3 and scale the CD4 and CD8 panels
CD3 <- cbind(CD4, CD8)
CD3Per <- mono$`CD3-CD19` - B$`B cells`
CD3def <- CD3Per*(CD3/100)


# remove unnecessary columns from the other panels
mono$CD45 <- NULL
mono$`CD3-CD19` <- NULL
B$CD45 <- NULL


# Join all the panels
flowper <- cbind(CD3def, B, mono)
rowSums(flowper[, which(!colnames(flowper) %in% c("CD4", "CD8", "B cells"))]) # it should roughly make up to 100
# CYFZ has more MDSCs
# 925L has more other types of helper T cells
# 4DUY has certain CD3 cell types that are neiter CD4, CD8, gd T cells or MAIT.


write.table(flowper, "CellsPercentages.txt", sep="\t")

pref_ord <- c("Progenitors", "pDCs","Basophils LD", "Neutrophils LD","mDCs", "Monocytes C", 
               "Monocytes I", "Monocytes NC", "B Naive",  "B NSM", "B SM", "B Ex", "Plasmablasts",  
               "T CD4 Naive","Tfh", "Tregs", "Th1" ,  "Th1/Th17","Th17" ,"Th2" ,"T CD4 EMRA",
               "T CD8 Naive", "T CD8 CM" ,"T CD8 EM", "T CD8 EMRA" ,"MAIT", "T gd Vd2","T gd non-Vd2", "NK" )
flowperSum <- data.frame(Mean= round(apply(t(flowper), 1, mean),2), SD= round(apply(t(flowper), 1, sd),2))
write.table(flowperSum[pref_ord,], "CellsPercentagesSum.txt", sep="\t", col.names = NA)

#flowper <- read.table("CellsPercentages.txt", sep ="\t", check.names = FALSE)
# prepare percentages of broader cell type groups. Run it after you run preprocess RNAseq data
setwd("~/Desktop/Work_temp/Deconvolution/")
load("RNA-seq/STAR_DGE.RData")
pheno <- STAR_DGE$samples
group1 <- as.character(pheno$group)
GRdf <- cbind.data.frame(group1, pheno[,grep("group[0-9]", colnames(pheno))], stringsAsFactors =F)
GRvec <- unique(unlist(GRdf))
GRvec2 <- GRvec[which(!GRvec %in% pheno$group)]

for(i in GRvec2){
  idGR<- which(apply(GRdf, 1, function(r) any(r %in% i)))
  cells <- as.character(unique(pheno$group[idGR]))
  flowper <- cbind(flowper, rowSums(flowper[,cells, drop=F]))
  colnames(flowper)[ncol(flowper)] <- i
}
save( flowper, file= "CellPercentageFULL.RData" )
