# process the featureCounts data file and merge it with annotations from the GENCODE GTF file as well as the sample annotations
setwd("~/Desktop/Work_temp/Deconvolution/RNA-seq")
# Sys.setlocale("LC_ALL", "en_US.ISO8859-1")
source("~/Desktop/Work_temp/Source code/NGS pipelines/other scripts/counts_to_tpm.R")

# load the libraries
libraries <- "plyr,reshape2,edgeR,futile.logger,rtracklayer,tximport"
libraries <- strsplit(libraries, ",")[[1]]
for (name in libraries) {
suppressPackageStartupMessages(library(name, character.only=T, warn.conflicts=F, quietly=T, verbose=F))
}
# Defining names and files
logfile <- "ProcessingReads"
outputSTAR <- "STAR"
outputkal <- "kallisto"
GeneralStats <- "qc/multiqc_data/multiqc_general_stats.txt"
countfile <- "star/counts/counts_largeOv.txt"
annotationGTF <- "genes_info/gencode.v26.annotation.ercc.gtf"
ERCCgtf <- "genes_info/ERCC92.gtf"    # this is for kallisto
pheno_data <- "pheno/pheno.txt" # make sure the names of the samples are under a column names sample
cellpercentages <- "../CellsPercentages.txt"
grouping <- "pheno/grouping_cell_types.txt"
# TEMRACD4 <- read.table("pheno/CD4TEMRA_RNAextr.txt", header = T, sep = "\t")
entrez <- read.table("genes_info/gencode.v26.metadata.EntrezGene")
GC_genes <- read.table("genes_info/GC_lengths_genes_v26.tsv")
GC_tx<- read.table("genes_info/GC_lengths_tx_v26.tsv")
# if you don't have info on the GC content, use the script "GTF2LengthGC.R" to retrieve it!  

# set up the loggers, wrap in invisible to prevent it from being printed
invisible(flog.threshold(DEBUG))
# remove the log file if it exists
if (file.exists(paste(logfile, ".log", sep=""))) {
  invisible(file.remove(paste(logfile, ".log", sep="")))
}
invisible(flog.appender(appender.tee(paste(logfile, ".log", sep=""))))
flog.info("LET'S SEE IF I GOT SOMETHING GOOD")

# read the phenotype file
pheno <- read.table(file=pheno_data, sep="\t", header=T, quote="", na.string="", as.is=T,check.names=F, comment.char="")
if (!"sample" %in% colnames(pheno)) {
  flog.fatal(paste("Unable to locate a sample column in the phenotype file"))
  stop()
}
# process the phenotype file for the RNA seq data
pheno <- pheno[order(pheno$sample),]
rownames(pheno) <- pheno$sample
# add total read count
stats <- read.table(GeneralStats, header = T)
stats_1 <- stats[grep("_1$", stats$Sample),]
rownames(stats_1) <- sub("_1$", "",stats_1$Sample)
pheno$tot_reads <- stats_1[pheno$sample,"FastQC_total_sequences"]

# calculate the add it to the pheno file
pheno$pg_cell <- (pheno$yield_ng/pheno$Cell_no.)*1000

#####################

# extract the median of the normalized value for each cell type
pg_median <- aggregate(pheno$pg_cell,by=list(pheno$group), median)
pheno2 <- merge(pheno, pg_median, by.x="group", by.y="Group.1")
pheno2 <- pheno2[order(pheno2$sample),]
pheno$pg_median <- pheno2$x

# # In case you want to change the PBMC yield with the one got it from the calculation of cell type percentages:
PBMCper <- read.table(cellpercentages, sep="\t",check.names = FALSE)
PBMCsub <- PBMCper[, which(!colnames(PBMCper) %in% c("CD4", "CD8", "B cells"))]
# group.first <- pheno[match(unique(pheno$group), pheno$group),]
# idx <- match(colnames(PBMCsub), group.first$group)
# totRNA <- t(t(PBMCsub) * group.first$pg_median[idx])
# PBMCyield1 <- rowSums(totRNA) / rowSums(PBMCsub)
# PBMCyield1 <-  PBMCyield1[paste0(names(PBMCyield1), "_PBMC") %in% pheno$Sample_Name]
# pheno$pg_cell[match(paste0(names(PBMCyield1), "_PBMC"), pheno$Sample_Name)] <- PBMCyield1
# # let's check the RNA yield of the four PBMC where I have individual RNA yield per cell type
indiv <- c("DZQV", "9JD4", "925L", "G4YW")
PBMCyield2<-c()
for(i in indiv){
  Pbmc1 <- pheno[grep(i , pheno$Sample_Name ), ]
  idx <- na.omit(match(colnames(PBMCsub), Pbmc1$group))
  idx2 <- match(Pbmc1$group[idx], colnames(PBMCsub))
  totRNA <- PBMCsub[i,idx2] * Pbmc1$pg_cell[idx]
  PBMCyield2[i] <- sum(totRNA) / sum(PBMCsub[i ,idx2])
}
# PBMCyield1
PBMCyield2
# # There might be the batch effect problem if you use these value, better to stick with the PBMC yield calculated with the median RNA yield of cell types
pheno$pg_median[which(pheno$group == "PBMCs")] <- median(PBMCyield2)

# create broader groups of cells
groups <- read.table(file=grouping, sep="\t", header=T, quote="", na.string="", as.is=T,check.names=F, comment.char="", row.names=1)

pheno$group2 <- groups[pheno$group,1]
pheno$group3 <- groups[pheno$group,2]
pheno$group4 <- groups[pheno$group,3]
pheno$group5 <- groups[pheno$group,4]
pheno$group6 <- groups[pheno$group,5]
pheno$group7 <- groups[pheno$group,6]
pheno$group8 <- groups[pheno$group,7]
pheno$group9 <- groups[pheno$group,8]
pheno$group10 <- groups[pheno$group,9]
###########################################################################
################ PROCESSING STAR+featureCounts READS ######################
# read the counts file
counts <- read.table(file=countfile, sep="\t", header=T, quote="", na.string="", as.is=T, check.names=F, comment.char="#")
flog.info(paste("Counts data read with", nrow(counts), "rows and", ncol(counts), "columns"))
counts <- rename(counts, c(Geneid="gene_id", Length="length"))
counts$Chr <- NULL  # removes the unwanted column
counts$Start <- NULL
counts$End <- NULL
counts$Strand <- NULL

# read the gencode file for the symbol and type
gencode <- import(annotationGTF, colnames=c("gene_id","gene_name","transcript_id","gene_type"), feature.type="transcript") # find the feature type in the third column of the gtf file
gencode <- data.frame(sapply(gencode@elementMetadata@listData,c), stringsAsFactors = F)
# if the readGFF function works you can also use:
# gencode <- readGFF(opt$gtf, filter=list(type="gene"), columns=character(0), tags=c("gene_id", "gene_name", "gene_type"))
ids <- match( gencode$transcript_id, entrez$V1)
gencode <- cbind(gencode, entrez$V2[ids])
colnames(gencode)[ncol(gencode)] <- "entrez_id"

gencGene <- gencode[!duplicated(gencode$gene_id),]
rownames(gencGene) <- gencGene$gene_id

flog.info(paste("GTF data read with", nrow(gencode), "genes"))

# prepare the gene level data (the feature Data)
fd <- counts[, c("gene_id", "length")]
rownames(fd) <- fd$gene_id   
fd$gene_name <- gencGene[rownames(fd), "gene_name"]   #be sure that you use the same annotation file that you used for featureCounts
fd$entrez_id <- gencGene[rownames(fd), "entrez_id"]  
fd$gene_type <- gencGene[rownames(fd), "gene_type"]
# add GC content
fd$gc <- GC_genes[rownames(fd), "GC"]

# Now finish to prepare the count data derived from featureCounts
sampleNames <- colnames(counts)[!colnames(counts) %in% c("gene_id", "length")]
counts1 <- counts[, sampleNames]
rownames(counts1) <- counts$gene_id
counts1 <- as.matrix(counts1)
sampleNames <- gsub("\\.bam$", "", sampleNames)
colnames(counts1) <- sampleNames

# check that all the samples exists in the phenotype file
if (!all(sampleNames %in% pheno$sample)) {
  flog.fatal("Unable to match all the samples to the phenotype file.")
  stop()
}
# check that all the genes in the counts files have data in the gencode file
if (!all(rownames(counts1) %in% rownames(fd))) {
  flog.fatal("Unable to match all the gene_ids in the count file to the GENCODE file.")
  stop()
}

# construct the DGEList
STAR_DGE <- DGEList(counts=counts1, genes=fd[rownames(counts1), ])
STAR_DGE$samples$group <- NULL
pheno2 <- merge(STAR_DGE$samples, pheno, by="row.names")
pheno2$Row.names <- NULL
rownames(pheno2) <- pheno2$sample
STAR_DGE$samples <- pheno2[rownames(STAR_DGE$samples), ]
flog.info("Finished constructing DGEList")

# remove the E26M samples
tokeep <- colnames(STAR_DGE)[which(!pheno$Sample_Name %in% c("E26M_CD4_EMRA"))]
STAR_DGE<-STAR_DGE[,tokeep]

# save the object in the DGElist format
STAR_filename <- paste(outputSTAR, "_DGE.RData", sep="")
save(STAR_DGE, file=STAR_filename)
flog.info(paste("Finished writing out", outputSTAR, "DGEList object to file:", STAR_filename))

# calculate tpm with original gene length
STAR.tpm <- tpm.standard(STAR_DGE$counts, STAR_DGE$genes$length)
save(STAR.tpm, file=paste0(outputSTAR, "_tpm.RData"))
# write.table(STAR.tpm, file=paste(outputSTAR, "_tpm.txt", sep=""), quote=F, sep="\t", na="", row.names=T, col.names=T)
flog.info("Finished writing out the STAR TPM data into a text file.")

# # Calculate the rpkm and add a prior count value to the counts scaled by the library size
# STAR.rpkm <- rpkm(STAR_DGE, normalized.lib.sizes=T, log=T,  prior.count=0.25)
# write.table(STAR.rpkm, file=paste(outputSTAR, "_rpkm.txt", sep=""), quote=F, sep="\t", na="", row.names=F, col.names=T)
# flog.info("Finished writing out the STAR RPKM data + prior into a text file.")

# calculate tpm with effective gene length
# STAR.tpm.el <- tpm_FL(STAR_DGE$counts, STAR_DGE$genes$length, as.numeric(STAR_DGE$samples$fragment_length))
# Ultimately I decided not to use it because this script removes the genes whereby the genelength is smaller than the fragment length.
# Moreover the gene length is also an average of different transcript, so it is already a manipulated measure.

# calculate tpm with a priorcount added and write it out
# STAR.prior<- addPriorCount(STAR_DGE$counts,  prior.count=0.25)
# if you want the adjusted library size use: exp(STAR.prior$offset) # be aware that because the prior count is very small (usually 0.25), in the log transformation this adjustment is going to be lost in the library size
# STAR1.tpm <- tpm.standard(STAR.prior[[1]], STAR_DGE$genes$length) 
# STAR.tpm.prior <- log2(STAR1.tpm)
# colnames(STAR.tpm.prior) <- colnames(STAR_DGE$counts)
# rownames(STAR.tpm.prior) <- rownames(STAR_DGE$counts)
# write.table(STAR.tpm.prior, file=paste(outputSTAR, "_tpm_prior.txt", sep=""), quote=F, sep="\t", na="", row.names=F, col.names=T)
# flog.info("Finished writing out the STAR TPM data + prior into a text file.")
# Ultimately I decided not to add a priorcount as it can introduce bias instead of actually adjusting the values

####################################################################
##################  PROCESS kallisto trasncripts ###################
# import kallisto counts
files <- file.path(getwd(), "kallisto", paste0(pheno$sample,"_kallisto"), "abundance.tsv")
names(files) <- pheno$sample
all(file.exists(files))
# the first time you run tximport on a new set of kallisto results use these two lines:
# kallisto_tx <- tximport(files, type="kallisto", txOut = T)
# save(kallisto_tx, file="kallisto_tximport.RData")
load("kallisto_tximport.RData")
# length is the effective length, counts are the effective counts, abundance is tpm

kallisto_tx$abundance <- kallisto_tx$abundance[,tokeep]
kallisto_tx$counts <- kallisto_tx$counts[,tokeep]
kallisto_tx$length <- kallisto_tx$length[,tokeep]

# handle feature data
ft_full <- strsplit(rownames(kallisto_tx$abundance),"\\|") # the result is equivalent to strsplit(rownames(kallisto_tx$abundance)[1],"|", fixed=T)
tx_fd <- data.frame(t(sapply(ft_full, function(x){c(x[1], x[2], x[3], x[4], x[5], x[6], x[7], x[8])})), stringsAsFactors = F) # you could also use matrix(unlist(ft_full), nrow = 8) if you are sure that every vector in the list has 8 elements
colnames(tx_fd) <- c("ENS_tx_ID", "ENS_gene_ID", "HAVANA_gene_ID", "HAVANA_tx_ID", "tx_name", "gene_name", "tx_length", "description")
# only manually annotate transcripts have an HAVANA ID, otherwise you will have a '-' 

# fix spikein naming
spikeins <- import(ERCCgtf, colnames=c("gene_id","transcript_id"), feature.type="exon") # find the feature type in the third column of the gtf file
spikedf <- cbind(elementMetadata(spikeins)@listData$transcript_id, elementMetadata(spikeins)@listData$gene_id, rep("-",92), rep("-",92),rep("-",92),rep("-",92), ranges(spikeins)@width ,rep("spike_in",92))
sub <- grep("ERCC", tx_fd[,1], value = T)
tx_fd[grep("ERCC", tx_fd[,1]), ] <- spikedf[match(sub, spikedf[,2]), ]
rownames(tx_fd) <- tx_fd[, "ENS_tx_ID"] # add rownames
tx_fd$gc <- GC_tx[rownames(tx_fd), "GC"] # add also GC content


ids <- match( tx_fd$ENS_tx_ID,gencode$transcript_id)
tx_fd<- cbind(tx_fd, gencode$entrez_id[ids])
colnames(tx_fd)[ncol(tx_fd)] <- "entrez_id"
# add the feature data to the list
kallisto_tx$genes <- tx_fd
rownames(kallisto_tx$abundance) <- rownames(kallisto_tx$genes)
rownames(kallisto_tx$counts) <- rownames(kallisto_tx$genes)
rownames(kallisto_tx$length) <- rownames(kallisto_tx$genes)


# Add also the phenotypic data to the list
kallisto_tx$samples <- STAR_DGE$samples[colnames(kallisto_tx$counts), ]
kallisto_tx$samples$lib.size <- colSums(kallisto_tx$counts)

# save the tximport object
kal_filename <- paste(outputkal, "_tx.RData", sep="")
save(kallisto_tx, file=kal_filename)
flog.info(paste("Finished writing out", outputkal, "tximport object to file:", kal_filename))
# load(kal_filename)
# write.table(kallisto_tx$abundance, file=paste(outputkal, "_tpm.txt", sep=""), quote=F, sep="\t", na="", row.names=F, col.names=T)
# # calculate tpm with a priorcount added and using the effective length and write it out
# kal.prior<- addPriorCount(kallisto_tx$counts,  prior.count=0.25)
# kallisto.tpm <- tpm.EL.matrix(kal.prior[[1]], kallisto_tx$length)
# kal.tpm.effLen <- log2(kallisto.tpm)
# colnames(kal.tpm.effLen) <- colnames(kallisto_tx$counts)
# rownames(kal.tpm.effLen) <- rownames(kallisto_tx$counts)
# write.table(kal.tpm.effLen, file=paste(outputkal, "_tpm_prior_effLen.txt", sep=""), quote=F, sep="\t", na="", row.names=F, col.names=T)
# flog.info("Finished writing out the kallisto TPM data + prior with effLen into a text file.")
# # Also here I decided not to add a prior count because it can introduce noise.
 
# this is the original length of each transcript
# file1 <- read.table(files[1], header = T)
# tx_length <- file1$length
# kallisto.tpm <- tpm.matrix(kallisto_tx$counts, tx_length)
# the tpm calculated with effective length gives better separation of the NK cells

# remember that for I used the effective length for kallisto. I hope this won't produce bias in the comparison with STAR

####################################################################
##################  PROCESS kallisto genes #########################
kal_gene <- summarizeToGene(kallisto_tx[1:4], kallisto_tx$genes[,c(1,2)])
kallisto_gene <- list(abundance = kal_gene$abundance[rownames(STAR_DGE$counts), colnames(STAR_DGE$counts)], 
                      counts = kal_gene$counts[rownames(STAR_DGE$counts), colnames(STAR_DGE$counts)], 
                      length = kal_gene$length[rownames(STAR_DGE$counts), colnames(STAR_DGE$counts)], 
                      countsFromAbundance = kal_gene$countsFromAbundance, 
                      samples = STAR_DGE$samples,
                      genes = STAR_DGE$genes)  
kal_filename <- paste(outputkal, "_gene.RData", sep="")
save(kallisto_gene, file=kal_filename)


########################################################################
############ PROCESS STAR values with effective length #################

# calculate tpm with original gene length
STAR.tpm2 <- tpm.standard(STAR_DGE$counts, kallisto_gene$length[rownames(STAR_DGE$counts),])
STAR_TPM_EL <- list( abundance = STAR.tpm2, length = kallisto_gene$length[rownames(STAR_DGE$counts),])
save(STAR_TPM_EL, file=paste0(outputSTAR, "_tpm_EL.RData"))
# write.table(STAR.tpm, file=paste(outputSTAR, "_tpm_EL.txt", sep=""), quote=F, sep="\t", na="", row.names=T, col.names=T)
flog.info("Finished writing out the STAR TPM EL data into a text file.")




