#+ setuppeeee, include=FALSE
# opts_chunk$set(echo=FALSE, dev = c('png'), fig.path = paste0("figure/", SaveName)) 
# opts_knit$set(progress = FALSE)

#' remember to assign a value between STAR, kal_tx, kal_genes to the SaveName variable before you start.
SaveName <- "kal_genes" # assign a value between STAR, kal_tx and kal_genes
source("~/Dropbox/_Deconvolution_paper/Src/4.RNAseqObj.R") # load the data that you intend analyze

# load the libraries
libraries <- "reshape2,edgeR,ggplot2,RColorBrewer,limma"
libraries <- strsplit(libraries, ",")[[1]]
for (name in libraries) {
  suppressPackageStartupMessages(library(name, character.only=T, warn.conflicts=F, quietly=T, verbose=F))
}

## If you want to remove DZQV
# sel <- !grepl("DZQV",pheno$Sample_Name)
# pheno <- pheno[sel, ]
# counts <- counts[,sel]
# dir.create("NO_DZQV", showWarnings = FALSE)
# setwd("NO_DZQV")

#'# Here I just use the voom with TPM values as the results are similar to the one obtained with counts and edgeR, but it is also faster.
#' voom + TPM 
#' voom + TPM + TMM scaling factors
#' voom + TPM + RNA yield per cell scaling factor
#' voom + TPM + RNA yield median scaling factor
#' voom + TPM + RNA yield median and TMM scaling factor
#' 
# In this version I also removed the function that creates boxplots, PCA and tsne on the data as it makes the script slow. If you want to use them again, see the version 1.

####################################################################################
################################## START HERE ######################################

#################################################################################### 
#build DGElist object with counts
d <- DGEList(counts = counts, group=pheno$group)
#'### Filtering 
#'# Filter only the genes that have at least 10 counts per million in 4 samples and remove ERCC by selecting only ENSembl genes or transcripts
CPM <- cpm(d$counts)
filter = apply(CPM, 1, function(x) length(x[x>4])>=3) # you can also use rowSums(cpm(d)>5) >= 3
filtGenes = rownames(d$counts)[filter][grep("^ENS", rownames(d$counts)[filter])]
d <- d[filtGenes, ]
dim(d)
# reset the library size
d$samples$lib.size <- colSums(d$counts)

filterENSonly <- rownames(counts)[grep("^ENS", rownames(counts))]
####################################################################################
# build DGElist object with TPM
TPM <- tpm.standard(counts[filtGenes,], as.matrix(len)[rownames(counts) %in% filtGenes,])
tpmFull <- tpm.standard(counts[filterENSonly,], as.matrix(len)[rownames(counts) %in% filterENSonly,])
tpm <- DGEList(counts = TPM, group=factor(pheno$group))

group1 <- as.character(pheno$group)
GRdf <- cbind.data.frame(group1, pheno[,grep("group[0-9]", colnames(pheno))], stringsAsFactors =F)
GRvec <- unique(unlist(GRdf))


  for(a in c(1:6)){
    if( a == 1){ # TPM 
      tpm1 <- tpm
      namenorm <- "tpm"
    }
    if( a == 2){ # TPM + TMM scaling factor
      tpm1 <- calcNormFactors(tpm, refColumn = 1)
      d1REFtmm <- tpm1$samples$norm.factors[1]
      PBMCnorm <- tpm1$samples$norm.factors[which(pheno$group == "PBMCs")]
      namenorm <- "tpmTMM"
    }
    if( a == 3){# TPM + RNA yield cell scaling factor
      tpm1 <- tpm
      yield_TMM <- ((1 /pheno$pg_cell)/ (1 /pheno$pg_cell[1])) * d1REFtmm
      yield_TMM[which(pheno$group == "PBMCs")] <- PBMCnorm
      tpm1$samples$norm.factors <- yield_TMM
      namenorm <- "tpmRNAcell"
    }
    if( a == 4){# TPM + RNA yield median scaling factor
      tpm1 <- tpm
      yield_TMM <- ((1 /pheno$pg_median)/ (1 /pheno$pg_median[1])) * d1REFtmm
      tpm1$samples$norm.factors <- yield_TMM
      namenorm <- "tpmRNAmed"
    }
    if( a == 5){# TPM + HK scaling factor
      tpm1 <- tpm
      load("HKfact.RData")  # this is created by a script that is subsequent to this one
      yield_HK <- ((1 /sc2)/ (1 /sc2[1])) * d1REFtmm
      tpm1$samples$norm.factors <- yield_HK
      namenorm <- "tpmHK"
    }
    if( a == 6){
      tpm1 <- tpm
      TMM_norm <- rep(1, ncol(tpm))
      celltypes <- unique(pheno$group[which(pheno$group != "PBMCs")])
      for(i in c(celltypes, "PBMCs")){
        idx <- which(pheno$group %in% i )
        temp <- calcNormFactors(tpm$counts[,idx])
        TMM_norm[idx] <-temp
      }
      tpm1$samples$norm.factors <- yield_TMM * TMM_norm   # TMM total = yield normalize * TMM
      namenorm <- "tpmRNAmedTMM"
    }
    
    dir.create(namenorm, showWarnings = FALSE)
    # save data
    tmm_norm <- sum(tpm1$counts[,1]) / (colSums(tpm1$counts) * tpm1$samples$norm.factors )
    tpm_norm <- sweep(tpm1$counts , MARGIN=2, tmm_norm, `*`)
    write.table(tpm_norm, paste0(namenorm, "/tpm_norm.txt"), sep="\t", col.names=NA)
    tpmFull_norm <- sweep(tpmFull , MARGIN=2, tmm_norm, `*`)
    write.table(tpmFull_norm, paste0(namenorm, "/tpmFull_norm.txt"), sep="\t", col.names=NA)
    # save ordered TPM file
    idGr <- sort(as.character(pheno$group), index.return=T)$ix
    TPMord <- tpm_norm[,idGr]
    colnames(TPMord) <- pheno$group[idGr]
    write.table(TPMord, paste0(namenorm, "/tpm_norm_ord.txt"), sep = "\t", col.names=NA)
    
    
    
    idPBMC <- which(apply(GRdf, 1, function(r) any(r %in% "PBMCs")))
    DE1 <- features[rownames(tpm1),]
    DE1 <- DE1[,c("gene_name","entrez_id"), drop=F]
    DE2 <- DE1
    DE3 <- DE1
    medGE <- DE1
    summary <- c()
    for(i in GRvec){
      if(i == "PBMCs") next
      idGR<- which(apply(GRdf, 1, function(r) any(r %in% i)))
      tpm1$samples$group <- "rest"
      tpm1$samples$group[idPBMC] <- "PBMCs"
      tpm1$samples$group[idGR] <- i
      tpm1$samples$group <- factor(tpm1$samples$group, levels= c("PBMCs", "rest", i))
      
      design.mat <- model.matrix(~ 0 + tpm1$samples$group)
      colnames(design.mat) <- levels(tpm1$samples$group)
      v <- voom(tpm1, design.mat, plot = FALSE)
      
      fit = lmFit(v, design.mat)
      fit = contrasts.fit(fit, c(0, -1, 1))
      fit = eBayes(fit)
      
      #####FROM HERE, select DEG only with adj pvalue<= 0.05
      # just to verify the number of differentially expressed genes:
      DEdt <- decideTests(fit, adjust.method="BH", p.value=0.05)
      summary <- cbind(summary, summary(DEdt))
      colnames(summary)[ncol(summary)] <- i
      
      # save differentially expressed genes with adj.pvalue less than 0.05:
      DEpval <- topTable(fit, sort.by = "p", p.value=0.05, n=nrow(fit))
      DE1 <- cbind(DE1, rep(0, nrow(DE1)))   # create a column with zeros
      colnames(DE1)[ncol(DE1)] <- i # give it a name
      if(nrow(DEpval) > 0 ) { DE1[rownames(DEpval), i] <- DEpval[,"logFC"] }   # fill it with log FC
      
      
      DEfc <- topTable(fit, sort.by = "p", p.value=1, n=nrow(fit)) # order by pvalue
      # DEfc <- round(DEfc,3)
      DEfcPOS <- DEfc[DEfc[,"logFC"] >0, "adj.P.Val", drop=F]  # now select only positive logFC
      DEfcNEG <- DEfc[DEfc[,"logFC"] <0, "adj.P.Val", drop=F] * -1  # now select only negative logFC
      DEfcTOT <- rbind(DEfcPOS[, ,drop=F], DEfcNEG[, ,drop=F]) # join them
      DE2 <- cbind(DE2, rep(0, nrow(DE2)))  # create new column with zeros
      colnames(DE2)[ncol(DE2)] <- i          # give it a name  
      DE2[rownames(DEfcTOT), i] <- DEfcTOT[,"adj.P.Val"]    # fill it with signed p-value  

      DE3 <- cbind(DE3, rep(0, nrow(DE3)))  # create new column with zeros
      colnames(DE3)[ncol(DE3)] <- i          # give it a name  
      DE3[rownames(DEfc), i] <- DEfc[,"logFC"]    # fill it with logFC
      
      med <- apply(tpm_norm[, idGR], 1, median)
      medGE <- cbind(medGE, med[rownames(medGE)])
      colnames(medGE)[ncol(medGE)] <- i
    }
    DE4 <- cbind(DE2[,c(1,2)], abs(DE2[,3:ncol(DE2)]))
    
    write.table(DE1, paste0(namenorm, "/DEtot.txt"), sep = "\t", col.names=NA)
    write.table(DE2, paste0(namenorm, "/DEdecFDR.txt"), sep = "\t", col.names=NA)
    write.table(DE3, paste0(namenorm, "/DEdecFC.txt"), sep = "\t", col.names=NA)
    write.table(DE4, paste0(namenorm, "/DEdecFDRabs.txt"), sep = "\t", col.names=NA)    
    write.table(medGE, paste0(namenorm, "/tpm_median.txt"), sep = "\t", col.names=NA)
    write.table(summary, paste0(namenorm,"/DE_summary.txt"), sep="\t",col.names = NA, row.names =  c("under-expressed", "no difference","over-expressed"))
  
    #############################################################
    ## Now check the variability of expression among individuals!!
    tpm1$samples$group <- substr(pheno$Sample_Name, 1,4)
    tpm1$samples$group[idPBMC] <- "PBMCs" # put name of individuals
    tpm1$samples$group <- factor(tpm1$samples$group)
    tpm2 <- tpm1
    design.mat <- model.matrix(~ 1+ tpm2$samples$group) # use the first individual as intercept. If you don't specificy the intercept, you have to specify the contrast matrix later on otherwise values will be compared against zero
    colnames(design.mat) <- levels(tpm2$samples$group)
    v <- voom(tpm2, design.mat, plot = FALSE)
    fit = lmFit(v, design.mat)
    fit = eBayes(fit)
    all <- topTable(fit, coef=2:4, n=nrow(fit))
    suball <- all[all$P.Value < 0.05, ]
    GeneNames <- features[rownames(suball), 3]
    write.table(cbind(suball, GeneNames), paste0(namenorm, "/DEGindividuals.txt"), sep = "\t", col.names=NA)    
    
    ####### In monocytes only ###############
    tpm1$samples$group <- substr(pheno$Sample_Name, 1,4)
    idNoMono <- which(apply(GRdf, 1, function(r) all(!r %in% "Monocytes NC+I")))
    tpm1$samples$group[idNoMono] <- "rest"
    # tpm1$samples$group[idPBMC] <- "PBMCs" # put name of individuals
    tpm1$samples$group <- factor(tpm1$samples$group)
    id <- which(tpm1$samples$group != "rest")
    tpm2 <- tpm1[,id]    # choose if you want to include all samples or only monocytes
    design.mat <- model.matrix(~ 1+ tpm2$samples$group) # use the first individual as intercept. If you don't specificy the intercept, you have to specify the contrast matrix later on otherwise values will be compared against zero
    colnames(design.mat) <- levels(tpm2$samples$group)
    v <- voom(tpm2, design.mat, plot = FALSE)
    
    fit = lmFit(v, design.mat)
    fit = eBayes(fit)

    all <- topTable(fit, coef=2:4, n=nrow(fit))
    suball <- all[all$P.Value < 0.05, ]
    GeneNames <- features[rownames(suball), 3]
    write.table(cbind(suball, GeneNames), paste0(namenorm, "/MonoI+NC_DEGindividuals.txt"), sep = "\t", col.names=NA)    
    
    ####### check monoNC vs MonoI ########
    tpm1$samples$group <- "rest"
    tpm1$samples$group[which(GRdf[,1] %in% "Monocytes I")] <- "MonoI"
    tpm1$samples$group[which(GRdf[,1] %in% "Monocytes NC")] <- "MonoNC"
    tpm1$samples$group <- factor(tpm1$samples$group)
    id <- which(tpm1$samples$group != "rest")
    tpm2 <- tpm1[,id] # I think it is better to remove all other samples
    design.mat <- model.matrix(~ 1+ tpm2$samples$group) # use the first individual as intercept. If you don't specificy the intercept, you have to specify the contrast matrix later on otherwise values will be compared against zero
    colnames(design.mat) <- levels(tpm2$samples$group)
    v <- voom(tpm2, design.mat, plot = FALSE)
    fit = lmFit(v, design.mat)
    fit = eBayes(fit)
    all <- topTable(fit, coef=2, n=nrow(fit),sort.by="P")
    suball <- all[all$P.Value < 0.05, ]
    GeneNames <- features[rownames(suball), 3]
    write.table(cbind(suball, GeneNames), paste0(namenorm, "/MonoDEG_IvsNC.txt"), sep = "\t", col.names=NA)   
    
    
    print(paste0("done with ", namenorm))
  }
  
