SaveName <- "kal_genes" # assign a value between STAR, kal_tx and kal_genes
source("~/Dropbox/_Deconvolution_paper/Src/4.RNAseqObj.R") # load the data that you intend analyze

libraries <- "plyr,reshape2,edgeR,ggplot2,Rtsne,RColorBrewer,EDASeq,stringr,sva,genefilter,gridExtra,dendextend,grid"
libraries <- strsplit(libraries, ",")[[1]]
for (name in libraries) {
  suppressPackageStartupMessages(library(name, character.only=T, warn.conflicts=F, quietly=T, verbose=F))
}

setwd("tpm")  # select which kind of normalization you want to use
TPMred <- read.table("tpm_norm.txt", sep="\t", row.names = 1, header=T)  

cells1 <- c("PBMCs", "Tfh",  "Tregs", "Th1" ,  "Th1/Th17","Th17" ,"Th2" , "T CD4 Naive", "T CD4 EMRA")
cells2 <- c("PBMCs","T gd Vd2","T gd non-Vd2" ,"MAIT", "T CD8 Naive", "TCD8 CM" ,"T CD8 EM", "T CD8 TEMRA")
cells3 <- c("PBMCs","Progenitors" , "B Naive",  "B NSM" , "B SM", "B Ex",  "Plasmablasts" )
cells4 <- c("PBMCs", "Monocytes C", "Monocytes I", "Monocytes NC","NK","mDCs","pDCs", "Neutrophils LD" ,  "Basophils LD" )

####### Let's start with Tsne and PCA
# plot
plotP <- function(cells, data, labs, legPos, textS){
  select <- which(pheno$group %in% cells)
  CellTypes <- factor(pheno$group[select], levels =cells)
  ggplot(data, aes(V1, V2)) + geom_point(colour =  "gray90") +  geom_point(data = subset(data[select,]), aes(color= CellTypes)) +  scale_colour_brewer(name="Cell types", palette = "Set1" ) + theme_bw() + labs(x = labs[1], y=labs[2]) + theme(
    text = element_text(size=textS[1]) ,
    legend.position=c(legPos[1],legPos[2]),legend.justification = c(legPos[1],legPos[2]), legend.key.height=unit(textS[2],"line"), legend.background = element_rect(size=0.5, linetype="solid", colour ="black")) # fill="lightblue", # this is for the background of the legend
}

#### tsne
ts <- Rtsne(t(log2(TPMred+1)))
tsne <- as.data.frame(ts$Y) # names of the variables are V1 and V2
labsTSNE <-c("","") #  c("t-SNE 1", "t-SNE 2") # modify column names here
legendTSNE <- c(1,0)  # modify legend position here
textTSNE <- c(18,0.9) # the first value is for text size, the second for legend spacing

##### PCA
fit <- prcomp(t(log2(TPMred+1)), scale = FALSE, center = TRUE)
#+ variancePCA, fig.width = 5, fig.height = 5
variance <- data.frame(pc=1:length(fit$sdev), variance=(fit$sdev^2) / sum(fit$sdev^2)) # this gives a scaled variance from 0 to 1
pvar <- ggplot(variance[1:20,], aes(factor(pc))) + geom_bar(aes(weight=variance*100)) + ylab("variance (%)") + xlab("component") + ggtitle("Variance per component") + theme_bw()+ theme(plot.margin=unit(c(3,1,0.5,0.5), "lines")) # change the first value of plot.margin for the upper spacing
pca <- as.data.frame(fit$x)[,1:2]
colnames(pca) <- c("V1", "V2")
labsPCA <- c("","") # c("PC 1", "PC 2") # modify column names here
legendPCA <- c(0,1)  # modify legend position here
textPCA <- c(15,0.8)
# levels(as.factor(pheno$group)) # see this to choose the cells

for(i in 1:4){
  if( i == 1){
    p1 <- plotP( cells1, tsne,labsTSNE,legendTSNE, textTSNE )
    p1b <- plotP( cells1, pca, labsPCA,legendPCA,textPCA )
  } 
  if( i == 2){
    p2 <- plotP( cells2, tsne,labsTSNE,legendTSNE,textTSNE ) 
    p2b <- plotP( cells2, pca, labsPCA,legendPCA,textPCA )
    }
  if( i == 3){
    p3 <- plotP( cells3, tsne,labsTSNE,legendTSNE, textTSNE) 
    p3b <- plotP( cells3, pca, labsPCA,legendPCA,textPCA)
  }  
  if( i == 4){
    p4 <- plotP( cells4, tsne,labsTSNE,legendTSNE, textTSNE ) 
    p4b <- plotP( cells4, pca, labsPCA,legendPCA,textPCA) #+ theme(plot.margin=unit(c(1,1,2,0.5), "lines")) # change the third value for the bottom spacing
  }  
}

gg <- grid.arrange(p1, p2,p3,p4, ncol=2)
ggsave("tsne_celltypes.pdf", gg, width = 11, height = 10)

ggb <- grid.arrange(p1b, p2b,p3b,p4b,pvar, ncol=1 ) # ,  top = textGrob("PCA analysis", gp = gpar(fontface = "bold", cex = 1.5)))
ggsave("pca_celltypes_long.pdf", ggb, width = 5.5, height = 25, limitsize = FALSE)

#ggb <- grid.arrange(p1b, p2b,p3b,p4b,pvar, nrow=3, layout_matrix = cbind(c(1,2,5), c(3,4,5)))
#ggsave("pca_celltypes.png", ggb, width = 13, height = 13)


########## Hierarchical clustering #############
labels <- as.factor(pheno$group)
lab2 <- paste(substr(pheno$Sample_Name,1,4), pheno$group)
# Hierarchical clustering
mat <- log2(TPMred+1)
colnames(mat) <- lab2
d <- dist(t(mat))
hc <- hclust(d)


dend <- hc %>% as.dendrogram
labHC <- dend %>% labels
cells <- substr(labHC, 6,nchar(labHC))
dend2 <- dend %>% set("labels", cells) %>%  set("labels_cex", 0.9) %>% set("labels_col", labelColors[cells])# %>%raise.dendrogram (-100)

don <- substr(labHC, 1,4)
colors <- brewer.pal(9, "Set1")
pal <- colorRampPalette(colors)
donorColors <- pal(length(unique(don)))
names(donorColors) <- unique(don)

dend3 <- dend2 %>% set("leaves_pch", 19) %>% set("leaves_cex", 1) %>% set("leaves_col", donorColors[don]) %>% set("branches_lwd", 2) 

ord <- c( "Neutrophils LD", "Basophils LD","Monocytes C", "Monocytes I", "Monocytes NC","mDCs","Tfh",  "Tregs", "Th1" ,  "Th1/Th17","Th17" ,"Th2" , "T CD4 Naive","NK","T CD4 EMRA","T gd Vd2","T gd non-Vd2" ,"MAIT", "T CD8 Naive", "T CD8 CM" ,"T CD8 EM", "T CD8 EMRA","pDCs", "Progenitors" , "B Naive",  "B NSM" , "B SM", "B Ex",  "Plasmablasts","PBMCs")

ordered <- unlist(lapply(1:length(ord), function(x) which(labels(dend3) %in% ord[x])))  

dend4 <- rotate(dend3,  ordered)

#png(filename = "HC.png",width = 1500, height = 5000, units = "px", pointsize = 70)
pdf(file = "HC.pdf",width = 7, height = 29, pointsize=15)
par(mar=c(4.5, 0, 0, 5))
plot(dend4,  xlab = "Height", horiz = TRUE,lwd=0.1) # main = "Immune cells clustering"
dev.off()



############## GC content 
data <- newSeqExpressionSet(counts, phenoData=AnnotatedDataFrame(
  data.frame(conditions=factor(pheno$group),
             row.names=pheno$sample)),
  featureData=AnnotatedDataFrame(features[rownames(TPM), ]))
fct <- levels(as.factor(pheno$group))
colors <- brewer.pal(9, "Set1")

# png(filename = "GC.png",width = 1000, height = 1000, units = "px", pointsize = 14)
pdf(file = "GC.pdf",width = 15, height = 15,pointsize = 14)
par(mfrow=c(2,2),cex.lab=1.5)
for (i in 1:4) {
  cells <- get(paste0("cells", i))
  select <- unlist(lapply(1:length(cells), function(x) which(pheno$group %in% cells[x]))) 
  colors2 <-colors[1:length(unique(pheno$group[select]))]
  names(colors2) <- unique(pheno$group[select])
  
  biasPlot(data[,select],"gc",ylim=c(0,3.5),col= colors2[as.character(pheno$group[select])], xlab="GC content")
  legend("topright", names(colors2),fill=colors2)
}
dev.off()
