SaveName <- "kal_genes" # assign a value between STAR, kal_tx and kal_genes
source("~/Dropbox/_Deconvolution_paper/Src/4.RNAseqObj.R") 
# load the libraries
libraries <- "dplyr,reshape2,ggplot2,RColorBrewer,ape,WGCNA"
libraries <- strsplit(libraries, ",")[[1]]
for (name in libraries) {
  suppressPackageStartupMessages(library(name, character.only=T, warn.conflicts=F, quietly=T, verbose=F))
}

# see the paper of Brawand 2011
namenorm <- dir(pattern="tpm")[1]
dir.create("phylogenesis", showWarnings = FALSE)

GR1 <- unique(as.character(pheno$group))
GR1 <- GR1[which(!GR1  %in% "PBMCs")]


MEDg <- read.table(paste0(namenorm, "/tpm_median.txt"), sep="\t", header=T, row.names = 1,check.names = FALSE) 
# MEDg$PBMCs <- NULL
MEDg <- MEDg[, GR1]
MEDgLOG <- log2(MEDg+1)

##### Plot tree
plot_TREE <- function(tree, boot, legend = T){
  EDcol <- rep("black", length(tree$edge.length))
  EDcol[which(tree$edge[,2] == which(tree$tip.label == "Progenitors"))] <- "darkorange4"
  Tcol <- rep("black", length(tree$tip.label))
  Tcol[which( tree$tip.label == "Progenitors")] <- "darkorange4"
  boot2<- boot
  boot2[which(boot2 >= 90)] <- "darkolivegreen4"
  boot2[which(boot2 >= 50 & boot2 < 90)] <- "darkgoldenrod1"
  boot2[which(boot2 < 50)] <- "coral1"
  plot(tree, root.edge =F,type = "phylogram", edge.color = EDcol, tip.color = Tcol, no.margin=T, cex=1) 
  nodelabels(pch =20, frame = "none", col = boot2)
  add.scale.bar() # remember that if you want you can add the length of the scale bar
 if(legend) legend(x="bottomright", legend= c(">=90",">=50 and <90", "<50") , col = c("darkolivegreen4", "darkgoldenrod1", "coral1"), bty = "n", pch=20, title = "Bootstrap",title.adj=0.15,y.intersp=0.8,x.intersp=0.3)
}

#plot(tree, root.edge =F,type = "phylogram", edge.color = EDcol, tip.color = Tcol, no.margin=T, cex=1) #, x.lim=1) 
#nodelabels(pch =20, frame = "none", col = boot2)


#d <- dist(t(MEDgLOG))
d <- 1 - cor(MEDgLOG, method = "spearman")  # method = "spearman"
hist(d,50)
tree <- nj(d)
tree <- root(tree, "Progenitors", resolve.root = TRUE)
f <- function(x) nj(1 - cor(t(x), method = "spearman"))
tr<-f(t(MEDgLOG))
boot <- boot.phylo(tr,  t(MEDgLOG), f,B = 100, rooted =TRUE)
pdf(file = "phylogenesis/phylo_spearman.pdf",width = 6, height = 8, pointsize=15)
plot_TREE(tree, boot)
dev.off()
#nodelabels(round(boot/10), adj = c(1.1, -0.1), frame = "none", cex = 0.5)


d <- 1 - cor(MEDgLOG)  # method = "pearson"
hist(d,50)
tree <- nj(d)
tree <- root(tree, "Progenitors", resolve.root = TRUE)
f <- function(x) nj(1 - cor(t(x), method = "pearson"))
tr<-f(t(MEDgLOG))
boot <- boot.phylo(tr,  t(MEDgLOG), f,B = 100, rooted =TRUE)
pdf(file = "phylogenesis/phylo_pearson.pdf",width = 7, height = 7, pointsize=15)
plot_TREE(tree, boot)
dev.off()


### 
d <- 1 - bicor(MEDgLOG, maxPOutliers = 0.1)  # method bicor
hist(d,50)
tree <- nj(d)
tree <- root(tree, "Progenitors", resolve.root = TRUE)
f <- function(x) nj(1 - bicor(t(x), maxPOutliers = 0.1))
tr<-f(t(MEDgLOG))
boot <- boot.phylo(tr,  t(MEDgLOG), f,B = 100, rooted =TRUE)
pdf(file = "phylogenesis/phylo_bicor.pdf",width = 7, height = 7, pointsize=15)
plot_TREE(tree, boot)
dev.off()
# ggtree(tree) + geom_label(aes(label=bootstrap),size=2) + geom_tiplab(size=2)
#geom_nodepoint(color="#b5e521", alpha=1/4, size=10)
#


# old versions!
# tree <- nj(1 - cor(MEDgLOG))
# tree <- root(tree, "Progenitors", resolve.root = TRUE)
# f <- function(x) nj(1 - cor(t(x)))
# boot <- boot.phylo(f(t(MEDgLOG)),  t(MEDgLOG), f,B = 100, rooted =TRUE)
# plot(tree, root.edge =F,type = "phylogram", edge.color =EDcol, tip.color = Tcol, no.margin=T, cex=0.6) 
# nodelabels(pch =20, frame = "none", col = boot2)
# # 
# # 
# tree <- nj(dist(t(MEDgLOG)))
# tree <- root(tree, "Progenitors", resolve.root = TRUE)
# plot(tree)
# f <- function(x) nj(dist(x))
# tr<-f(t(MEDgLOG))
# boot <- boot.phylo(tr,  t(MEDgLOG), f,B = 100, rooted =TRUE)
# nodelabels(round(boot), adj = c(1.1, -0.1), frame = "none", cex = 0.5)
# 
# 



# 
# library("phangorn")
# tree <- upgma( 1 - cor(MEDgLOG))
# tree <- root(tree, "Progenitors")
# plot(tree)
# 
# tree <- upgma( 1 - cor(MEDgLOG, method = "spearman"))
# tree <- root(tree, "Progenitors")
# plot(tree)
# 
# tree <- upgma( dist(t(MEDgLOG)))
# tree <- root(tree, "Progenitors")
# plot(tree)


# library(ggtree)
# tree <- apeBoot(tree, boot)
# ggtree(tree) +geom_nodepoint(color=c("#b5e521","black"), alpha=1/2, size=2)+ geom_tiplab()
