SaveName <- "kal_genes" # assign a value between STAR, kal_tx and kal_genes
source("~/Dropbox/_Deconvolution_paper/Src/4.RNAseqObj.R") 
options(stringsAsFactors = FALSE)

# load the libraries
libraries <- "dplyr,reshape2,edgeR,ggplot2,RColorBrewer,e1071,preprocessCore,epiR,ComplexHeatmap,WGCNA,circlize"
libraries <- strsplit(libraries, ",")[[1]]
for (name in libraries) {
  suppressPackageStartupMessages(library(name, character.only=T, warn.conflicts=F, quietly=T, verbose=F))
}

namenorm <- dir(pattern="tpm")[1]
TPM <- read.table(paste0(namenorm, "/tpm_norm.txt"), sep="\t", header=T, row.names = 1,check.names = FALSE)
TPM <- TPM[!duplicated(features[rownames(TPM),]$gene_name),]
DEFC <- read.table(paste0(namenorm, "/DEdecFC.txt"), sep="\t", header=T, row.names = 1,check.names = FALSE)
DEFC <- DEFC[rownames(TPM),]
DEFDR <- read.table(paste0(namenorm, "/DEdecFDR.txt"), sep="\t", header=T, row.names = 1,check.names = FALSE)
DEFDR <- DEFDR[rownames(TPM),]
TPM_gene_names <- TPM
rownames(TPM_gene_names) <- DEFC$gene_name

dir.create(paste0(namenorm,"/heatmapDE"), showWarnings = FALSE)
# setwd("heatmapDE")

GRdf <- cbind.data.frame(as.character(pheno$group), pheno[,grep("group[0-9]", colnames(pheno))], stringsAsFactors =F)
GRvec <- unique(unlist(GRdf))
GR1 <- colnames(DEFC)[which(colnames(DEFC) %in% GRvec)]

DEFDR1 <- DEFDR[,c("gene_name", GR1)]
DEFC1 <- DEFC[,c("gene_name", GR1)]
DEFDRmelt <- melt(DEFDR1, id.vars = c("gene_name"))
DEFCmelt <- melt(DEFC1, id.vars = c("gene_name"))
FC1 <- DEFCmelt  %>% filter(value > 4 ) %>% arrange(desc(value)) # using the fold change of 2 seems to improve the results respect to smaller fold changes
FDR1 <- DEFDRmelt  %>% filter(value > 0 & value < 0.005 ) %>% arrange(value) # if you change the threshowld with 0.2 you get worse results
FC1 <- FC1[!duplicated(FC1$gene_name),]
combFC1 <- paste(FC1$gene_name, FC1$variable)
combFDR1 <- paste(FDR1$gene_name, FDR1$variable)
FDR1 <- FDR1[combFDR1 %in% combFC1, ]
dim(FDR1)

idORD <- order(match(pheno$group, pref_ord))

TPMs <-log2(TPM_gene_names[as.character(FDR1$gene_name),idORD]+1)
colnames(TPMs) <- pheno$group[idORD]

### FIND CLUSTERS
hc <- hclust(dist(TPMs))
modules = cutreeDynamic(dendro = hc, distM = as.matrix(dist(TPMs)),cutHeight =40, method="hybrid",deepSplit = 0, pamStage = F, pamRespectsDendro = FALSE, minClusterSize = 45)
STcolors <- standardColors(100)[!grepl("light", standardColors(100))]
module.colours = labels2colors(modules, colorSeq = STcolors)
names(modules) <- module.colours
pdf(paste0(namenorm,"/heatmapDE/clusteringDE.pdf"))
plotDendroAndColors(hc, module.colours, 'Module colours', dendroLabels = FALSE, hang = 0.03, addGuide = TRUE, guideHang = 0.05, main='',marAll = c(1, 5, 0.5, 0.5))
dev.off()

### temporary heatmap
TPMs2 <- TPMs[modules!= 0,]
dim(TPMs2)
TPMs2[TPMs2>8] <- 8
h1<- Heatmap(TPMs2, name = "expr", col = c(rev(brewer.pal(8, "Spectral"))), width = unit(13, "cm"),
             cluster_columns = FALSE, split= factor(modules[modules!= 0]), column_names_gp = gpar(col = labelColors[colnames(TPMs2)], fontsize = 4),
             show_row_names=F, row_dend_side = "left",  row_title_rot=0,  row_title_gp = gpar(fontsize = 5),
             heatmap_legend_param = list(title = "Log2 TPM", at = c(0, 2, 4, 6,8), labels = c("0", "2", "4", "6","8"),  
                                         width = unit(0, "mm") ), use_raster = T, raster_device ="png" ) 
pdf(paste0(namenorm,"/heatmapDE/heatmapDE_temp2.pdf") ,width = 10, height = 8)
draw(h1, main_heatmap = "expr", cluster_rows = TRUE) #, padding=padding)
dev.off()


### EIGENGENE MODULES
MEs0 = moduleEigengenes(t(TPMs), colors = module.colours, excludeGrey = FALSE)$eigengenes
MEs = orderMEs(MEs0)
MEDiss = dist(MEs)
METree = hclust(as.dist(MEDiss), method = 'average')
pdf(paste0(namenorm,"/heatmapDE/eigengeneDE.pdf"))
plotEigengeneNetworks(MEs, "Eigengene adjacency heatmap", marHeatmap = c(3,4,2,2), plotDendrograms = FALSE, xLabelsAngle = 90)
dev.off()

############### MERGING ################
#sizeGrWindow(7, 6)
plot(METree, main = "Clustering of module eigengenes",xlab = "", sub = "")
MEDissThres = 0.03
# Plot the cut line into the dendrogram
abline(h=MEDissThres, col = "red")
merge = mergeCloseModules(t(TPMs), module.colours, cutHeight = MEDissThres, verbose = 3, corFnc = bicor, corOptions = list(maxPOutliers = 0.1))
mergedColors = merge$colors;
mergedMEs = merge$newMEs;

### PLOT NEW CLUSTERING
# sizeGrWindow(12, 9)
#png("heatmap/signedALL_clustSize60.png",  width = 480, height = 480)
plotDendroAndColors(hc, cbind(module.colours, mergedColors),
                    c("Dynamic Tree Cut", "Merged dynamic"),
                    dendroLabels = FALSE, hang = 0.03,
                    addGuide = TRUE, guideHang = 0.05)
dev.off()
# Construct numerical labels corresponding to the colors
colorOrder = c("grey", STcolors)  
modulesMER = match(mergedColors, colorOrder)-1   # compare it with colnames(MEsMerged) 
names(modulesMER) <- mergedColors


######## START ANALYSIS FOR HEATMAP #############
mod <- modules  # choose between modules (for original splitting) or modulesMER (for merged modules)
modSize <- table(mod)
# I like better the original splitting :)

### ENRICHMENT ANALYSIS
# check if the GO terms are updated. Then run this:
load( file="~/Desktop/Work_temp/Source code/Database and annotation/GOids.RData")
GOids2 <- GOids[GOids[,"hgnc_symbol"] %in% rownames(TPMs), ] # select only GO terms that are associated with out list of genes
GOids2 <- subset(GOids2, namespace_1003=="biological_process") # select only biological processes
GOids2 <- subset(GOids2, go_id %in% names(table(GOids2$go_id)[table(GOids2$go_id) >2])) # remove GO terms that have only 2 genes
GOterms <- unique(GOids2[,'go_id'])[unique(GOids2[,'go_id']) != ""]
universe <- unique(GOids2[,"hgnc_symbol"])
res<-c()
for(m in unique(mod)){
  modGenes <- rownames(TPMs)[mod==m]
  modGenes <- modGenes[modGenes %in% universe]
  notmodGenes <- universe[!universe %in%  modGenes]
  temp <-c()
  for(i in GOterms){
    vect <- unique(subset(GOids2,go_id == i)[,"hgnc_symbol"])
    GSmod <- sum(vect %in% modGenes) # differentially expressed in the Gene set
    GSnotmod <- sum(vect %in% notmodGenes)
    RESTmod <- length(modGenes) - GSmod
    RESTnotmod <- length(notmodGenes) - GSnotmod
    contigency.matrix <- rbind( c(GSmod, GSnotmod), c(RESTmod, RESTnotmod) )
    pval <- signif(fisher.test( contigency.matrix, alternative="greater")$p.value, 3) 
    temp <- rbind(temp, c(m,modSize[as.character(m)], i, GSmod, pval))
  }
  temp<- cbind(temp, p.adjust(temp[,5], method = "fdr"))
  temp <- temp[order(as.numeric(temp[,5])),]
  temp <- temp[as.numeric(temp[,5])<0.05,]
  res <- rbind(res, temp)
  print(paste("done with",m))
}
GOids3 <- GOids2[!duplicated(GOids2$go_id),]
rownames(GOids3) <- GOids3$go_id
GOres<- cbind(res, GOids3[res[,3],5:6])
colnames(GOres) <- c("module", "module size", "GO term","ModGenesInTerm", "p value","FDR", "GO name", "GO domain")

GOres2 <- filter(GOres, module !=0)
GOres2$`GO domain` <- NULL
write.table(GOres2, file = paste0(namenorm,"/heatmapDE/GOresDE.txt"), sep = "\t") # in case you do it again, remove rownames and consider changing column names

# GOres2 <- GOres %>% group_by(module) %>% filter(row_number() <= 10)  # in case you want only 10
# GOres2$fdr <- NULL # remove fdr value and add it just in case they ask

# using Complex heatmap
# split <- as.character(modules[idFULL])
# h1<- Heatmap(TPMs[idFULL,],cluster_columns = FALSE, cluster_rows = T, show_row_names=F, row_dend_side = "left", split= split,row_title_rot=0, row_title_gp = gpar(fontsize = 5))
# h1

# ORDER MODULES
TPMsmelt <- melt(cbind(TPMs, mod), id.vars = "mod")
ave <- TPMsmelt %>% group_by(mod, variable) %>% summarize(mean_mod = mean(value, na.rm = TRUE))
ave <- filter(ave, variable != "PBMCs" & mod != 0)
aveC <- dcast(ave, mod ~ variable,value.var= "mean_mod" )
ord <- apply(aveC, 1, function(x) which.max(x[-1]))
ordMod <- aveC$mod[order(ord)]


# process enrichment data
GOres3 <- filter(GOres2, as.numeric(ModGenesInTerm) > 1 | grepl("defense|immune|response|CD8|interferon|nerve|platelet|immunoglobulin|antimicrobial|neutrophil|recombination|differentiation|interleukin",`GO name`))
# minModsize <- min(GOres2$modSize)
GOres4 <- GOres3 %>% group_by(module) %>% filter(nchar(`GO name`) < 36) %>%   #filter(!duplicated(`p value`))
          filter(row_number() %in% 1:min(n(), 5, max(2, round(as.numeric(`module size`)/min(modSize))))) # choose the number of GO term to report
unique(GOres4$module)
GOres4$module <- as.numeric(GOres4$module)
GOres4$titleHM <- paste(GOres4$`GO term`, GOres4$`GO name`, "\n ")
GOHeat <- c()
for(i in ordMod){
  temp <- subset(GOres4, module ==i)
  GOHeat <- rbind(GOHeat, c(i, paste(temp$titleHM, collapse="" )))
}
GOHeat[,2] <- sub(" \n $", "", GOHeat[,2])

# prepare the expression values to report
idNOzero <- which(mod != 0)
TPMheat <- TPMs[idNOzero,]
TPMheat[TPMheat>8] <- 8

# how to split the rows
while(any(duplicated(GOHeat[,2]))){
  GOHeat[duplicated(GOHeat[,2]),2] <-paste0(GOHeat[duplicated(GOHeat[,2]),2]," ")
}
names(idNOzero) <- GOHeat[match(mod[idNOzero], GOHeat[,1]), 2]
split <- factor(names(idNOzero),levels= GOHeat[,2])
# split<- factor(mod[idNOzero], levels=GOHeat[,1]) # if you don't want the GO use this

# process genes to report 
FDR2 <- cbind(FDR1, mod)
FDR2 <- filter(FDR2, mod != 0)
write.table(FDR2, file = paste0(namenorm,"/heatmapDE/genesDE.txt"), sep = "\t")
minMod3 <- min(table(FDR2$mod))/3
FDR3 <- FDR2  %>% arrange(value)%>% group_by(mod) %>% filter(row_number() %in% 1:min(10,round(n()/minMod3)) ) # choose the max number of genes
genes<- FDR3$gene_name 
pos <- match(genes, rownames(TPMheat))
genelabs <- paste0(FDR3$gene_name, " (", FDR3$variable,")") # how to label them


#colors for modul legend
colAnnot <- names(mod)[match(ordMod, mod)]
names(colAnnot) <- ordMod

# do the heatmap
ha <- rowAnnotation(Modules = as.character(mod[idNOzero]),col=list(Modules = colAnnot), 
                    annotation_legend_param = list(Modules = list(ncol = 2, title = "Modules", title_position = "topcenter")), width = unit(2, "mm"))
h1<- Heatmap(TPMheat, name = "expr", split= split, col = c(rev(brewer.pal(8, "Spectral"))), width = unit(13, "cm"), 
             cluster_columns = FALSE,  column_names_gp = gpar(col = labelColors[colnames(TPMheat)], fontsize = 4),
             show_row_names=F, row_dend_side = "left",  row_title_rot=0,  row_title_gp = gpar(fontsize = 5), 
             heatmap_legend_param = list(title = "Log2 TPM", at = c(0, 2, 4, 6,8), 
                                         labels = c("0", "2", "4", "6","8"),   width = unit(0, "mm") ),
             use_raster = T, raster_device ="png" ) +  
             rowAnnotation(link = row_anno_link(at = pos, labels = genelabs, labels_gp =gpar(fontsize = 4), 
                                                link_width =  unit(5, "mm")), width = unit(5, "mm") + max_text_width("aaaaaaaaaaa"))
#padding = padding = unit(c(0.2,25, 0.2, 25), "cm") #bltr
pdf(paste0(namenorm,"/heatmapDE/heatmapDE.pdf") ,width = 10, height = 8)
#par(oma = c(5,40,4,2))
draw(ha+h1, main_heatmap = "expr", cluster_rows = TRUE) #, padding=padding)
dev.off()


# 
# h1<-Heatmap(TPMheat, col = c(rev(brewer.pal(8, "Spectral"))),
#             cluster_columns = FALSE,  column_names_gp = gpar(col = labelColors[colnames(TPMheat)], fontsize = 5),split= split , 
#             show_row_names=F, row_dend_side = "left",  row_title_rot=0,  row_title_gp = gpar(fontsize = 5), 
#             heatmap_legend_param = list(title = "log TPM", at = c(0, 2, 4, 6,8), labels = c("0", "2", "4", "6","8")) ) + 
#   rowAnnotation(link = row_anno_link(at = pos, labels = genelabs, labels_gp =gpar(fontsize = 5), link_width =  unit(5, "mm")),  width = unit(0.1, "mm") + max_text_width(genelabs)) 
# h2 <- Heatmap(mod[idNOzero], name = "modules", width = unit(1, "mm"), col=names(mod[idNOzero]),  show_row_names = F,  cluster_rows = FALSE,row_title_rot=0, show_heatmap_legend = FALSE) # this only works with numberic matrices!! REMEMBER THIS
# h1+h2




# other complex heatmaps options
# col = colorRamp2(seq(0,14,2), c(rev(brewer.pal(8, "Spectral")) # you have to use the package circlize
# heatmap_legend_side = "bottom"  # add it to the draw function
# nrow = 1, title_position = "topcenter",  # add it to the heatmap_legend_param
# row_names_side = "left"


# using ggplot
# prova <- TPMs[idMERGED[1:10], 1:20]
# split <- names(idMERGED)[1:10]
# prova2 <- cbind(prova, split, names = rownames(prova))
# PRmelt <- melt(prova2, id.vars = c("names","split"))
# cols <- sub( "\\.[0-9]*","",PRmelt$variable)
# PRmelt$var2 <- factor(cols, levels= unique(cols))
# 
# 
# p <- ggplot(PRmelt, aes(x=variable, y=names)) + geom_tile(aes(fill = value)) + facet_grid(split ~ var2 , scales='free', space="free")+ theme(panel.spacing = unit(0.05, "lines"), strip.background = element_rect(fill=c("white")))
# p




#heatmap.2(as.matrix(TPMs))
  # take differentially expressed genes. Select the ones with a low Pvalue lower than 0.05  in more than one cell type!
  
  # Just prepare the code to produce the heatmap also and if you have to you can change the filtering later on.






##### other ways to select genes
# # select genes with an expression of at least of 500 in two samples of one group
# genes <-c()
# for(cols in GR1){
#   temp <- apply(TPM[,pheno$group == cols], 1, function(x) sum(x > 500) >= 2)
#   genes <- c(genes, rownames(TPM)[temp])
# }
# 
# # now select only genes that are larger than 500 in at least present in any 4 samples 
# temp <- apply(TPM[genes, pheno$group %in% GR1], 1, function(x) sum(x > 500) >= 6)
# head(TPM[genes[temp],])