# IMPORTANT: play with the option deep split of the dynamic cut tree.
# let's not use the PAMStage option becauase as input I already use a lot of genes. It is fine if I get a lot of genes unassigned. You might try using PAMstage too, but it can be more difficult to interpret the results and it probably wouldn't add anything significant to the analysis

SaveName <- "kal_genes" # assign a value between STAR, kal_tx and kal_genes
source("~/Dropbox/_Deconvolution_paper/Src/4.RNAseqObj.R") 
options(stringsAsFactors = FALSE);
# load the libraries
libraries <- "dplyr,reshape2,ggplot2,RColorBrewer,limma,WGCNA,ComplexHeatmap"
libraries <- strsplit(libraries, ",")[[1]]
for (name in libraries) {
  suppressPackageStartupMessages(library(name, character.only=T, warn.conflicts=F, quietly=T, verbose=F))
}

namenorm <- dir(pattern="tpm")[1]
TPM <- read.table(paste0(namenorm, "/tpm_norm.txt"), sep="\t", header=T, row.names = 1,check.names = FALSE)
TPM <- TPM[!duplicated(features[rownames(TPM),]$gene_name),]
DEFDR <- read.table(paste0(namenorm, "/DEdecFDR.txt"), sep="\t", header=T, row.names = 1,check.names = FALSE)
DEFDR <- DEFDR[rownames(TPM),]
TPM_gene_names <- TPM
rownames(TPM_gene_names) <- features[rownames(TPM),]$gene_name

dir.create(paste0(namenorm,"/heatmapCOEXP"), showWarnings = FALSE)

#transform the data
# TPMv = voom(TPM)$E
TPMl <- log2(TPM_gene_names+1)
#select the necessary columns
TPMct <-TPMl[ , which(!pheno$group == "PBMCs")]


#filter genes
TPMct2 <- TPMct[rowSums(TPMct) > 50, ]
TPMct3 <- TPMct2[apply(TPMct2, 1, function(x) length(x[x>5])>=3.5),]    # ncol(TPMl2)*0.05)
# TPMct3 = TPMct2[order(apply(TPMct2,1,mad), decreasing = T)[1:5000],]
#variancedatExpr=as.vector(apply(as.matrix(TPMct3),1,var, na.rm=T))
#expr = t(TPMct3[variancedatExpr>0.5 ,])
expr = t(TPMct3)
dim(expr)

# rename columns of original matrix
idORD <- order(match(pheno$group, pref_ord))
TPMs <-TPMl[colnames(expr),idORD]
colnames(TPMs) <- pheno$group[idORD]


# # choose the beta value
# powers <- c(c(1:10), seq(from = 12, to=20, by=2))
# sft <- pickSoftThreshold(expr, powerVector = powers, verbose = 5,networkType = "signed", corFnc = "bicor", corOptions = "maxPOutliers = 0.1")
# plot(sft$fitIndices[,1], -sign(sft$fitIndices[,3])*sft$fitIndices[,2],
#      xlab='Soft Threshold (power)',ylab='Scale Free Topology Model Fit,signed R^2',
#      type='n', main = paste('Scale independence'));
# text(sft$fitIndices[,1], -sign(sft$fitIndices[,3])*sft$fitIndices[,2],
#      labels=powers,cex=1,col='red'); abline(h=0.90,col='red')
beta <- 6 # 6 for unsigned and 12 for signed

# calculate the adjacency matrix
## s = bicor(expr, maxPOutliers = 1) # use biweight midcorrelation as it is more robust to outliers
s = abs(cor(expr, method="spearman"))
##s = abs(cor(expr))
adj <- s^beta
##adj = adjacency(expr, power = beta, type="unsigned", corFnc = "cor")
##adj = adjacency(expr, power = beta, type="unsigned", corFnc = "bicor", corOptions = "maxPOutliers = 0.01")
TOM = TOMsimilarity(adj, TOMType="unsigned")
rownames(TOM) <- rownames(adj)
colnames(TOM) <- colnames(adj)
save(TOM, file=paste0(namenorm,"/heatmapCOEXP/TOM.RData"))
load(paste0(namenorm,"/heatmapCOEXP/TOM.RData")) # practically you could start from here

# calculate dissimilarity matrix
# w = 1-adj
w = 1-TOM  # dissimilarity matrix
# w = 1-TOMsimilarityFromExpr(expr, power = 12)  # maybe this is faster

# clustering
geneTree = hclust(as.dist(w), method = 'average')
#sizeGrWindow(12,9)
#plot(geneTree, xlab="", sub="", main = "Gene clustering on TOM-based dissimilarity", labels = FALSE, hang = 0.04);

#finding modules  # you could also try to limit the number of genes for one cluster, but probably you have to do it manually
modules = cutreeDynamic(dendro = geneTree, distM = w, method="hybrid",deepSplit = 1,pamStage = F, pamRespectsDendro = FALSE, minClusterSize = 35) # if you increase deepSplit you obtain more clusters
STcolors <- standardColors(100)[!grepl("light", standardColors(100))]
module.colours = labels2colors(modules, colorSeq = STcolors)
names(modules) <- module.colours
pdf(paste0(namenorm,"/heatmapCOEXP/clusteringCOEXP.pdf"))
plotDendroAndColors(geneTree, module.colours, 'Module colours', dendroLabels = FALSE, hang = 0.03, addGuide = TRUE, guideHang = 0.05, main='')
dev.off()

# explore modules similarity and merge similar modules
MEs0 = moduleEigengenes(expr, colors = module.colours, excludeGrey = FALSE)$eigengenes
MEs = orderMEs(MEs0)
MEDiss = 1-cor(MEs)
METree = hclust(as.dist(MEDiss), method = 'average')
pdf(paste0(namenorm,"/heatmapCOEXP/eigengeneCOEXP.pdf"))
plotEigengeneNetworks(MEs, "Eigengene adjacency heatmap", marHeatmap = c(3,4,2,2),
                      plotDendrograms = FALSE, xLabelsAngle = 90)
dev.off()
sizeGrWindow(7, 6)
plot(METree, main = "Clustering of module eigengenes",xlab = "", sub = "")
MEDissThres = 0.20
# Plot the cut line into the dendrogram
abline(h=MEDissThres, col = "red")
merge = mergeCloseModules(expr, module.colours, cutHeight = MEDissThres, verbose = 3, corFnc = bicor, corOptions = list(maxPOutliers = 0.1))
mergedColors = merge$colors;
mergedMEs = merge$newMEs;
ncol(mergedMEs)

sizeGrWindow(12, 9)
#png("heatmap/signedALL_clustSize60.png",  width = 480, height = 480)
pdf(paste0(namenorm,"/heatmapCOEXP/clusteringCOEXPMerged.pdf"))
plotDendroAndColors(geneTree, cbind(module.colours, mergedColors),
                    c("Dynamic Tree Cut", "Merged dynamic"),
                    dendroLabels = FALSE, hang = 0.03,
                    addGuide = TRUE, guideHang = 0.05)
dev.off()
MEsMerged0 = moduleEigengenes(expr, colors = mergedColors, excludeGrey = FALSE)$eigengenes
MEsMerged = orderMEs(MEsMerged0)
pdf(paste0(namenorm,"/heatmapCOEXP/eigengeneCOEXPMerged.pdf"))
plotEigengeneNetworks(MEsMerged, "Eigengene adjacency heatmap", marHeatmap = c(3,4,2,2), plotDendrograms = FALSE, xLabelsAngle = 90)
dev.off()
# Construct numerical labels corresponding to the colors
colorOrder = c("grey", STcolors)  
modulesMER = match(mergedColors, colorOrder)-1   # compare it with colnames(MEsMerged) 
names(modulesMER) <- mergedColors

######## START ANALYSIS FOR HEATMAP #############
mod <- modules  # choose between modules (for original splitting) or modulesMER (for merged modules)
modSize <- table(mod)
#order according to hierarchical clustering
ordcolMod <- sub("^ME","",colnames(MEs)) # choose between MEs (for original splitting) or MEsMerged (for merged modules)
ordcolMod <- ordcolMod[ordcolMod != "grey"]
ordMod <- match(ordcolMod, colorOrder)-1  

# ORDER MODULES ACCORDING TO CELL TYPES
# TPMsmelt <- melt(cbind(TPMs, mod), id.vars = "mod")
# ave <- TPMsmelt %>% group_by(mod, variable) %>% summarize(mean_mod = mean(value, na.rm = TRUE))
# ave <- filter(ave, variable != "PBMCs" & mod != 0)
# aveC <- dcast(ave, mod ~ variable,value.var= "mean_mod" )
# ord <- apply(aveC, 1, function(x) which.max(x[-1]))
# ordMod <- aveC$mod[order(ord)]


### ENRICHMENT ANALYSIS
# check if the GO terms are updated. Then run this:
load( file="~/Desktop/Work_temp/Source code/Database and annotation/GOids.RData")
GOids2 <- GOids[GOids[,"hgnc_symbol"] %in% rownames(TPMs), ] # select only GO terms that are associated with out list of genes
GOids2 <- subset(GOids2, namespace_1003=="biological_process") # select only biological processes
GOids2 <- subset(GOids2, go_id %in% names(table(GOids2$go_id)[table(GOids2$go_id) >2])) # remove GO terms that have only 2 genes
GOterms <- unique(GOids2[,'go_id'])[unique(GOids2[,'go_id']) != ""]
universe <- unique(GOids2[,"hgnc_symbol"])
res<-c()
for(m in unique(mod)){
  modGenes <- rownames(TPMs)[mod==m]
  modGenes <- modGenes[modGenes %in% universe]
  notmodGenes <- universe[!universe %in%  modGenes]
  temp <-c()
  for(i in GOterms){
    vect <- unique(subset(GOids2,go_id == i)[,"hgnc_symbol"])
    GSmod <- sum(vect %in% modGenes) # differentially expressed in the Gene set
    GSnotmod <- sum(vect %in% notmodGenes)
    RESTmod <- length(modGenes) - GSmod
    RESTnotmod <- length(notmodGenes) - GSnotmod
    contigency.matrix <- rbind( c(GSmod, GSnotmod), c(RESTmod, RESTnotmod) )
    pval <- signif(fisher.test( contigency.matrix, alternative="greater")$p.value, 3) 
    temp <- rbind(temp, c(m,modSize[as.character(m)], i, GSmod, pval))
  }
  temp<- cbind(temp, p.adjust(temp[,5], method = "fdr"))
  temp <- temp[order(as.numeric(temp[,5])),]
  temp <- temp[as.numeric(temp[,5])<0.05,]
  res <- rbind(res, temp)
  print(paste("done with",m))
}
GOids3 <- GOids2[!duplicated(GOids2$go_id),]
rownames(GOids3) <- GOids3$go_id
GOres<- cbind(res, GOids3[res[,3],5:6])
colnames(GOres) <- c("module", "module size", "GO term","ModGenesInTerm", "p value","FDR", "GO name", "GO domain")

GOres2 <- filter(GOres, module !=0)
GOres2$`GO domain` <- NULL
write.table(GOres2, file = paste0(namenorm,"/heatmapCOEXP/GOresCOEXP.txt"), sep = "\t") # in case you do it again, remove rownames and consider changing column names

# using Complex heatmap
# split <- as.character(modules[idFULL])
# h1<- Heatmap(TPMs[idFULL,],cluster_columns = FALSE, cluster_rows = T, show_row_names=F, row_dend_side = "left", split= split,row_title_rot=0, row_title_gp = gpar(fontsize = 5))
# h1


# process enrichment data
GOres3 <- filter(GOres2, as.numeric(ModGenesInTerm) > 2 | grepl("defense|immune|response|CD8|interferon|nerve|platelet|immunoglobulin|antimicrobial|neutrophil|recombination|differentiation|interleukin",`GO name`))
GOres4 <- GOres3 %>% group_by(module) %>% filter(nchar(`GO name`) < 36) %>%   #filter(!duplicated(`p value`))
  filter(row_number() %in% 1:min(n(), 3, round(as.numeric(`module size`)/(min(modSize)+40)))) # choose the number of GO term to report
unique(GOres4$module)
GOres4$module <- as.numeric(GOres4$module)
GOres4$titleHM <- paste(GOres4$`GO term`, GOres4$`GO name`, "\n ")
GOHeat <- c()
for(i in ordMod){
  temp <- subset(GOres4, module ==i)
  GOHeat <- rbind(GOHeat, c(i, paste(temp$titleHM, collapse="" )))
}
GOHeat[,2] <- sub(" \n $", "", GOHeat[,2])

# prepare the expression values to report
idNOzero <- which(mod != 0)
TPMheat <- TPMs[idNOzero,]
TPMheat[TPMheat>8] <- 8

# how to split the rows
while(any(duplicated(GOHeat[,2]))){
  GOHeat[duplicated(GOHeat[,2]),2] <-paste0(GOHeat[duplicated(GOHeat[,2]),2]," ")
}
names(idNOzero) <- GOHeat[match(mod[idNOzero], GOHeat[,1]), 2]
split <- factor(names(idNOzero),levels= GOHeat[,2])
# split2<- factor(mod[idNOzero], levels=GOHeat[,1]) # if you don't want the GO use this
# PROCESS GENES TO REPORT
con <- data.frame(cbind( gene_name=rownames(adj), value=round(rowSums(adj)), mod = mod))
write.table(con, file = paste0(namenorm,"/heatmapCOEXP/connectivity.txt"), sep = "\t")
con$value <-as.numeric(con$value)
pdf(paste0(namenorm,"/heatmapCOEXP/connectivityCOEXP.pdf"), width = 14,height = 7)
par(mar=c(5,4,2,1))
# boxplot(con$value ~ factor(con$mod, levels =c(ordMod, "0")) , pch=20, xlab="Module", ylab="Gene connectivity", col = c(ordcolMod, "grey"), medcol="white") # the module 1 contains more hub genes than all the remaining modules
boxplot(con$value[which(con$mod != 0)] ~ factor(con$mod[which(con$mod != 0)], levels =ordMod) , pch=20, xlab="Module", ylab="Gene connectivity", col = c(ordcolMod), medcol="white") # the module 1 contains more hub genes than all the remaining modules
dev.off() # maybe add the filling with colors  ,col=c("gold","darkgreen")
# Find HUBS among unassigned genes
# first method
# conINTER <- filter(con, mod == 0)
# conINTER2 <- conINTER  %>% arrange(desc(as.numeric(value)))
# source("C:/Users/GianniMonaco/Desktop/Work/Source code/Outlier_detection.R")
# idHUBs0 <- OutIQR(as.numeric(conINTER2$value), side = "pos")  # outlier values
# a second method that calculates the correlation with module eigengenes 
# adjINTER <- bicor(expr[,conINTER$gene_name], MEsMerged, maxPOutliers = 0.1)
# adjINTER[adjINTER<0] <- 0
# adjINTER2 <- sort(rowSums(adjINTER),decreasing=T)
# Find INTRAMODULAR genes
# aa<-chooseTopHubInEachModule(expr,power=12, modules, type="signed",corFnc = "bicor", corOptions = "maxPOutliers = 0.1")
conINTRA <- filter(con, mod != 0)
boxplot(as.numeric(conINTRA$value) ~ conINTRA$mod) # the module 1 contains more
minMod <- min(table(conINTRA$mod))
conINTRA2 <- conINTRA  %>% arrange(desc(value)) %>% group_by(mod) %>% filter(row_number() %in% 1:min(10,round(n()/minMod)) )  # 1:min(5, n())) # choose the max number of genes
genes<- conINTRA2$gene_name 
pos <- match(genes, rownames(TPMheat))
genelabs <- paste0(conINTRA2$gene_name, " (", conINTRA2$value,")") # how to label them
dev.off()


#colors for modul legend
colAnnot <- names(mod)[match(ordMod, mod)]
names(colAnnot) <- ordMod


# do the heatmap
ha <- rowAnnotation(Modules = as.character(mod[idNOzero]),col=list(Modules = colAnnot), annotation_legend_param = list(Modules = list(ncol = 2, title = "Modules", title_position = "topcenter")), width = unit(2, "mm"))
h1<- Heatmap(TPMheat, name = "expr", split = split, col = c(rev(brewer.pal(8, "Spectral"))), width = unit(13, "cm"), 
             cluster_columns = FALSE,  column_names_gp = gpar(col = labelColors[colnames(TPMheat)], fontsize = 4),
             show_row_names=F, row_dend_side = "left",  row_title_rot=0,  row_title_gp = gpar(fontsize = 5), 
             heatmap_legend_param = list(title = "Log2 TPM", title_position = "topcenter", at = c(0, 2, 4, 6,8), 
                                         labels = c("0", "2", "4", "6","8"),   width = unit(0, "mm") ), 
             use_raster = T, raster_device ="png") + 
  rowAnnotation(link = row_anno_link(at = pos, labels = genelabs, labels_gp =gpar(fontsize = 4), link_width =  unit(5, "mm")), width = unit(5, "mm") + max_text_width("aaaaaa"))
#padding = padding = unit(c(0.2,25, 0.2, 25), "cm") #bltr
pdf(paste0(namenorm,"/heatmapCOEXP/heatmapCOEXP.pdf"),width = 10.3, height = 8)
#par(oma = c(5,40,4,2))
draw(ha+h1, main_heatmap = "expr", cluster_rows = TRUE) #, padding=padding)
dev.off()

# h2<- Heatmap(TPMheat, split= split, col = c(rev(brewer.pal(8, "Spectral"))),
#              cluster_columns = FALSE,  column_names_gp = gpar(col = labelColors[colnames(TPMheat)], fontsize = 5),
#              show_row_names=F, row_dend_side = "left",  row_title_rot=0,  row_title_gp = gpar(fontsize = 5,col =ordcolMod), 
#              heatmap_legend_param = list(title = "log TPM", at = c(0, 2, 4, 6,8), labels = c("0", "2", "4", "6","8")) ) + 
#   rowAnnotation(link = row_anno_link(at = pos, labels = genelabs, labels_gp =gpar(fontsize = 5), link_width =  unit(5, "mm")),  width = unit(0.1, "cm") + max_text_width(genelabs))
# h2








##  build the blockwise network - I think your computer can manage a single block analysis though
# bwnet = blockwiseModules(TPMs, maxBlockSize = 2000,
#                          power = 12, TOMType = "signed", minModuleSize = 20,
#                          reassignThreshold = 0, mergeCutHeight = 0.25,
#                          numericLabels = TRUE, corType ="bicor",maxPOutliers =0.1,
#                          saveTOMs = TRUE,
#                          saveTOMFileBase = "ImmuneCellsTOM-blockwise",
#                          verbose = 3)

