SaveName <- "kal_genes" # assign a value between STAR, kal_tx and kal_genes
source("~/Dropbox/_Deconvolution_paper/Src/4.RNAseqObj.R") 
# load the libraries
libraries <- "dplyr,reshape2,edgeR,ggplot2,RColorBrewer,e1071,preprocessCore,epiR"
libraries <- strsplit(libraries, ",")[[1]]
for (name in libraries) {
  suppressPackageStartupMessages(library(name, character.only=T, warn.conflicts=F, quietly=T, verbose=F))
}

namenorm <- dir(pattern="tpm")[1]
dir.create("complexity", showWarnings = FALSE)

# 
group1 <- as.character(pheno$group)
GRdf <- cbind.data.frame(group1, pheno[,grep("group[0-9]", colnames(pheno))], stringsAsFactors =F)
# idx <- which(!pheno$group %in% "PBMCs")
# GRdfnoPBMC <- GRdf[idx,]
# filter = apply(counts, 1, function(x) length(x[x>0])>=1)
# filtGenes = rownames(counts)[filter][grep("^ENS", rownames(counts)[filter])]
# countMEA <- c()
# lenMEA <- c()
# for(i in colnames(GRdfnoPBMC)){
#   cells <- GRdfnoPBMC[, i]
#   ctemp <- t(aggregate(t(counts[filtGenes,idx]), by=list(cells), mean))
#   ltemp <- t(aggregate(t(len[filtGenes,idx]), by=list(cells), mean))
#   countMEA <- cbind(countMEA, ctemp)
#   lenMEA <- cbind(lenMEA, ltemp)
# }
# 
# colnames(countMEA) <- countMEA[1, ]
# countMEA2 <- countMEA[-1, ]
# countMEA2<-apply(countMEA2,2,as.numeric)
# colnames(lenMEA) <- lenMEA[1, ]
# lenMEA2 <- lenMEA[-1, ]
# lenMEA2<-apply(lenMEA2,2,as.numeric)
# 
# TPMfull <- tpm.standard(countMEA2, as.matrix(lenMEA2))
# rownames(TPMfull) <- rownames(counts[filtGenes, ])
# TPMfull <- TPMfull[,!duplicated(colnames(TPMfull))]
# save(TPMfull, file="complexity/TPMmean_full.RData")
load("complexity/TPMmean_full.RData")


GR1 <- unique(as.character(pheno$group))
GR1 <- GR1[which(!GR1  %in% "PBMCs")]

# calculate the cumulative distribution for all the cell types
MEAg2 <- as.matrix(TPMfull[, GR1])
MEAg3 <- apply(MEAg2, 2, function(x) c(0, cumsum(sort(x, decreasing = T))))
for(col in colnames(MEAg3)){
  MEAg3[duplicated(MEAg3[,col]) , col] <-NA
}
MEAg4 <- stack(as.data.frame(MEAg3))
MEAg4 <- MEAg4[!is.na(MEAg4[,1]),]
n <- c()
tab <- table(MEAg4$ind)
for(col in unique(MEAg4$ind)){
  n <-c(n, 1:tab[col])
}
MEAg4$len <- n

# take a representative cumulative distribution for each group
GR2 <- c("T cells","B cells","Monocytes", "Progenitors", "Plasmablasts", "Basophils LD", "Neutrophils LD", "NK", "DCs")
repct <- c() # choose the representative cell type
for(ct in GR2){
  temp <- apply(GRdf, 1, function(x) any(ct %in% x))
  temp2 <- tab[unique(GRdf[temp,1])] - median(tab[unique(GRdf[temp,1])])
  repct <- c(repct, names(which.min(abs(temp2))))
}
names(GR2) <- repct

MEAgPLOT <- MEAg4[MEAg4$ind %in% repct, ]
newNames <- GR2[as.character(MEAgPLOT$ind)]
MEAgPLOT$ind <- factor(newNames, levels = GR2[names(sort(tab[repct], decreasing=T))])

colors <- brewer.pal(9, "Set1")
pal <- colorRampPalette(colors)
labelColors <- pal(length(GR2))
names(labelColors) <- GR2

value80percent <- max(MEAg4$values) *0.8

p <- ggplot(data = MEAgPLOT, aes(y=values, x=len, group=ind, colour=ind) ) + geom_line(size=0.7)  + 
  scale_color_manual(values=labelColors[as.character(unique(MEAgPLOT$ind))]) + 
  labs(x= "number of genes", y="cumulative TPM value",colour = "Cell types") + theme_classic() + 
  theme(legend.position=c(1,0),legend.justification = c(1,0) , text = element_text(size=8), axis.text = element_text(color = "black"), 
        legend.background = element_rect(size=0.2, linetype="solid", colour ="black"),axis.line = element_line(color="black",size=0.2), 
        panel.grid.major.y = element_line(colour = "azure3"),  legend.key.height=unit(0.5,"line")) +
  geom_hline(yintercept=value80percent, linetype="dashed", color = "red") +
  annotate("text", x=33000, y=value80percent*1.025, label="80% cumulative TPM",color = "red", size=1.5) 
p
ggsave("complexity/cumulative curves.pdf", p , width=3, height=3)

# genes that take up 80 % of the total gene expression
MEAg5<- MEAg4 %>% group_by(ind) %>% filter(values < value80percent)  %>% summarize(n())

ngenes <- MEAg5$`n()`
names(ngenes) <- MEAg5$ind

pdf("complexity/80perGenes.pdf", width=5, height=3, pointsize = 8)
par(mar=c(6,4,1,0),  oma= c(0,0,0,0)) # you could try to use the ps option here
barplot(sort(ngenes, decreasing = T), las=2, col = "darkcyan", ylab = "Number of genes 80% cum. TPM", ylim=c(0,5000), lwd=0.8,border = NA) # , mgp=c(2.8,1,0))
dev.off()


# for(col in colnames(MEAg3)){
#   id <- which(MEAg4$ind %in% col)
#   gap.plot(MEAg4$len[id],MEAg4$value[id], gap = c(15000, 25000), gap.axis = "x",  col= labelColors[col], type="l",lty=1)
# }

# an other way to look at the data
# MEAg2 <- as.matrix(TPMMEA[,GR1])
# MEAg2[which(MEAg2 >0 & MEAg2<10)] <-1
# MEAg2[which(MEAg2 >=10 & MEAg2 < 100)] <-2
# MEAg2[which(MEAg2 >=100 & MEAg2 < 1000)] <-3
# MEAg2[which(MEAg2 >=1000 & MEAg2 < 10000)] <-4
# MEAg2[which(MEAg2 >=10000)] <-5
# tab <-apply(MEAg2, 2, table)
# tab2 <- tab[-1,]
# barplot(tab2, las =2)
# 
# 


# Alternative way to aggregate
# #TPM2 <- data.frame(t(TPM), group = pheno$group) # alternative way with dplyr
# #TPMMEA <- t(TPM2 %>% group_by(group) %>% summarise_each(funs(MEAian)))

# select the rows that are 
# cells <- unlist(apply(GRdf,1, function(x) unique(x[which(x %in% GR1)])))

