libraries <- strsplit("edgeR,ggplot2", ",")[[1]]
for (name in libraries) {
  suppressPackageStartupMessages(library(name, character.only=T, warn.conflicts=F, quietly=T, verbose=F))
}

# library("sleuth")
SaveName <- "kal_genes" # assign a value between STAR, kal_tx and kal_genes
source("~/Dropbox/_Deconvolution_paper/Src/4.RNAseqObj.R") 

setwd("~/Desktop/Work_temp/Deconvolution/RNA-seq")

# in case you want to check the extra CD4 TEMRA
# TEMRACD4extra <- (TEMRACD4$yield_ng/TEMRACD4$Cell_no.)*1000
# TEMRACD4All <- c(pheno$norm[which(pheno$Description == "CD4_TEMRA")], TEMRACD4extra)
# boxplot(pheno$norm[which(pheno$Description == "CD4_TEMRA")], TEMRACD4extra, TEMRACD4All)

# prepare data with no PBMCs
noPBMC <- which(pheno$group != "PBMCs")
noPBMC_ord <- which(pref_ord != "PBMCs")
ord <- pref_ord[noPBMC_ord]
pheno0 <- pheno[noPBMC,]
TPM0 <- TPM[,noPBMC]
counts0 <- counts[, noPBMC]
Donor <- substr(pheno0$Sample_Name, 1,4)
gr <- factor(pheno0$group, levels = ord)

# do the plot
dat <- data.frame(gr=gr, yi =pheno0$pg_cell )
pdf("TRIzol_yield.pdf", width=10, height=6)
ggplot(dat, aes(x=gr, y=yi, color = Donor)) +geom_boxplot(color="black",outlier.shape = NA)+
  geom_jitter(position=position_jitter(0.2))+ scale_color_brewer(palette="Set1")+ theme_bw() + 
  theme(text = element_text(size=20),axis.text.x = element_text(angle = 90, hjust = 1, color= "black"), legend.position="none") + 
  xlab("Cell type")+ ylab("Quant-iT / FACS (pg per cell)") # picogram
# if you don't want the legend remove legend.position="none"
dev.off()



TMM <- calcNormFactors(TPM0, reference=1)
yTMM <- 1/(TMM)
yTMMn <- (yTMM/0.8409894) * 0.35753333 # I don't remember why I did this!
dat2 <- data.frame(gr=gr, yi =yTMM )
pdf("TMM_yield.pdf", width=10, height=6)
ggplot(dat2, aes(x=gr, y=yi, color = Donor)) +geom_boxplot(color="black",outlier.shape = NA)+
  geom_jitter(position=position_jitter(0.2))+ scale_color_brewer(palette="Set1")+ theme_bw() + 
  theme(text = element_text(size=20),axis.text.x = element_text(angle = 90, hjust = 1, color= "black"), legend.position="none") + 
  xlab("Cell type")+ ylab("1/TMM scaling factor")
dev.off()



# after you run the deconvolution script you can run this:
load( file=paste0("process_", SaveName, "/optimizationSigMat.RData"))
dat2 <- stack(as.data.frame(optFull))
dat2 <- dat2[!is.na(dat2$values), ]
dat2$ind <- factor(dat2$ind, unique(names(GR0[match(levels(gr), GR0)])))
dat3 <- filter(dat2, ind!="Progenitors")
pdf("OptRNAy.pdf", width=10, height=6)
ggplot(dat3, aes(x=ind, y=values)) +geom_boxplot(color="black",outlier.shape = NA)+
  geom_jitter(position=position_jitter(0.2))+ theme_bw() + 
  theme(text = element_text(size=20),axis.text.x = element_text(angle = 90, hjust = 1, color= "black"), legend.position="none") +
  xlab("Cell type")+ ylab("Deconvolution scaling factor")
dev.off()


# 
# 
# results <- c()
# for(a in 1:2){
#   if (a == 1){   # all donors
#     STAR_DGE1 <- STAR_DGE[, which(pheno$group != "PBMCs")]
#     pheno1 <- STAR_DGE1$samples
#     STAR1 <-calcNormFactors(STAR_DGE1)
#     res_temp1 <- cor(STAR1$samples$norm.factors * STAR1$samples$lib.size,  STAR1$samples$lib.size/pheno1$pg_cell)
#    # norm <- norm_factors(STAR_DGE1$counts)
#    # res_temp2 <- cor(norm, STAR_DGE1$samples$lib.size/pheno1$pg_cell)
#     cols <- "cor_ALLcells"
#   }
#   if (a == 2){    # no infected donor
#     STAR_DGE1 <- STAR_DGE[, grep("DZQV|PBMC", pheno$Sample_Name, invert = T)]
#     pheno1 <- STAR_DGE1$samples
#     STAR1 <-calcNormFactors(STAR_DGE1)
#     res_temp1 <- cor(STAR1$samples$norm.factors * STAR1$samples$lib.size,  STAR1$samples$lib.size/pheno1$pg_cell)
#    # norm <- norm_factors(STAR_DGE1$counts)
#    # res_temp2 <- cor(norm, STAR_DGE1$samples$lib.size/pheno1$pg_cell)
#     cols <- "cor_ALLcells"
#   }
#   
#   for(groupCells in c("GR1", "GR2", "GR3", "GR4")){ # minor, main and essential
#     if(groupCells  == "GR1"){ groups <- pheno1$group }
#     if(groupCells  == "GR2"){ groups <- pheno1$group2 }
#     if(groupCells  == "GR3"){ groups <- pheno1$group3 }
#     if(groupCells  == "GR4"){ groups <- pheno1$group4 }
#     
#     for(i in unique(groups)){
#       idx <-  which(groups == i)
#       STAR1 <-calcNormFactors(STAR_DGE1[, idx])
#       res_temp1 <- c(res_temp1, cor(STAR1$samples$norm.factors * STAR1$samples$lib.size,  STAR1$samples$lib.size/pheno1$pg_cell[idx]))
#     #  norm <- norm_factors(STAR_DGE1$counts[, idx])
#     #  res_temp2 <- c(res_temp2,  cor(norm, STAR_DGE1$samples$lib.size[idx]/pheno1$pg_cell[idx]))
#       cols <- c(cols, paste0("cor_",i))
#     }
#   }
#   results <- cbind(results, res_temp1)
# }
# 
# rownames(results) <- cols
# results <- results[match(unique(cols), cols), ]
# # colnames(results) <- c("tmm_full", "deseq_full", "tmm_noInfDon", "deseq_noInfDon")
# colnames(results) <- c("tmm_full", "tmm_noInfDon")
# write.table(results, file= "cor_Yield_TMM.txt", sep="\t", col.names=T)
# 
# pheno <- STAR_DGE$samples
# # Let's check the correlation between tmm and the yield of PBMCs calculated from the median yield of cell types and cell types percentages
# PBMCper <- read.table("../CellsPercentages.txt", sep="\t",check.names = FALSE)
# PBMCsub <- PBMCper[, which(!colnames(PBMCper) %in% c("CD4", "CD8", "B cells"))]
# group.first <- pheno[match(unique(pheno$group), pheno$group),]
# idx <- match(colnames(PBMCsub), group.first$group)
# totRNA <- t(t(PBMCsub) * group.first$pg_median[idx])
# PBMCyield1 <- rowSums(totRNA) / rowSums(PBMCsub)
# PBMCyield1 <-  PBMCyield1[paste0(names(PBMCyield1), "_PBMC") %in% pheno$Sample_Name]
# pheno$pg_cell[match(paste0(names(PBMCyield1), "_PBMC"), pheno$Sample_Name)] <- PBMCyield1
# 
# # calculate TMM
# STAR_DGE2 <- calcNormFactors(STAR_DGE[, which(pheno$group == "PBMCs")])
# # compare the different normalization methods
# lib<- pheno$lib.size[ which(pheno$group == "PBMCs")]
# cor(pheno$pg_cell[ which(pheno$group == "PBMCs")] * lib, lib/STAR_DGE2$samples$norm.factors)
# 
#     


### OLD plots
######### exploring the amount of RNA produced per cell #############
# pdf("picogram_per_cell.pdf", width=12, height=8)
# 
# par(mar=c(7,4,3,1))
# noPBMC <- which(pheno$group != "PBMCs")
# boxplot(pheno$pg_cell[noPBMC] ~pheno$group[noPBMC],las=2, ylab= "RNA yield per cell in Picograms")
# stripchart(pheno$pg_cell[noPBMC] ~pheno$group[noPBMC], vertical = TRUE, 
#            method = "jitter", add = TRUE, pch = 20, col = 'lightblue')
# # let's have a look to a reshaped dataset to see if there is some batch effect (remove PBMCs and CD4 TEMRA as they have been extracted in too many different days)
# noCD4TEMRA <- which(pheno$group != "CD4 TEMRA")
# noPBMC_noCD4TEMRA <- intersect(noPBMC, noCD4TEMRA)
# # summary(lm(pheno$pg_cell[noPBMC_noCD4TEMRA] ~ pheno$Date_sorting[noPBMC_noCD4TEMRA]))
# # reshape(pheno[noPBMC,c("group", "Date_Extracted", "pg_cell", "Date_sorting")], idvar = "group", direction="wide", timevar = "Date_sorting", v.names="pg_cell")
# boxplot(pheno$pg_cell[noPBMC_noCD4TEMRA] ~ pheno$Date_sorting[noPBMC_noCD4TEMRA], 
#         las=2, main = "Is there some batch effect?", ylab= "RNA yield per cell in Picograms", xlab= "Day of sorting")
# # let's check with the boxplot 
# pTemp <- pheno[noPBMC_noCD4TEMRA,]
# boxplot(pTemp$pg_cell ~ pTemp$group,las=2, ylab= "RNA yield per cell in Picograms", 
#         main= "points colored by date of sorting")
# for(i in 1:4){
#   subpTemp <- subset(pTemp, Date_sorting == levels(factor(pTemp$Date_sorting))[i])
#   stripchart(subpTemp$pg_cell ~ subpTemp$group, vertical = TRUE, method = "jitter", 
#              add = TRUE, pch = 20, col = i+1)
# }
# legend(x=-1,y=2.5,legend = levels(factor(pTemp$Date_sorting)), bty="n",text.col = 2:5, y.intersp=1)
# dev.off()