SaveName <- "kal_genes" # assign a value between STAR, kal_tx and kal_genes
source("~/Dropbox/_Deconvolution_paper/src/4.RNAseqObj.R") 
source("~/Desktop/Work_temp/Source code/deconvolution/LLS.R")
source("~/Desktop/Work_temp/Source code/deconvolution/CIBERSORT.R")


# load the libraries
libraries <- "dplyr,reshape2,edgeR,ggplot2,RColorBrewer,e1071,preprocessCore,epiR"
libraries <- strsplit(libraries, ",")[[1]]
for (name in libraries) {
  suppressPackageStartupMessages(library(name, character.only=T, warn.conflicts=F, quietly=T, verbose=F))
}

# select and create folders
namenorm <- dir(pattern="tpm")[1]
dir.create("deconOpt", showWarnings = FALSE)

TPM <- read.table(paste0(namenorm, "/tpm_norm.txt"), sep="\t", header=T, row.names = 1,check.names = FALSE) 

# get median expression values
MEDg <- read.table(paste0(namenorm, "/tpm_median.txt"), sep="\t", header=T, row.names = 1,check.names = FALSE) 
TPM <- TPM[rownames(MEDg) ,]

# load flow cytometry data
load(file= "../../CellPercentageFULL.RData")

# Get DE data on FDR and FC
DEFDR <- read.table(paste0(namenorm, "/DEdecFDR.txt"), sep="\t", header = T, check.names = FALSE, stringsAsFactors = F,row.names = 1)
DEFDR <- DEFDR[rownames(MEDg) ,]
DEFC <- read.table(paste0(namenorm, "/DEdecFC.txt"), sep="\t", header = T, check.names = FALSE, stringsAsFactors = F,row.names = 1)
DEFC <- DEFC[rownames(MEDg) ,]

# prepare new rownames with Gene names
dup <- DEFDR$gene_name[duplicated(DEFDR$gene_name)]
idx <- DEFDR$gene_name %in% dup
DEFDR$gene_name[idx] <- paste0(DEFDR$gene_name[idx], "_",rownames(DEFDR)[idx]) 
DEFC$gene_name <- DEFDR$gene_name
rownames(MEDg) <- DEFDR$gene_name
rownames(TPM) <- DEFDR$gene_name

# save TPM with new rownames
save(TPM, file="TPM_geneName.RData")
save(MEDg, file="MedianGeneName.RData")
save(DEFDR, DEFC, file="DE_geneName.RData")

# split the TPM file
TPMct <-TPM[ , which(!pheno$group == "PBMCs")]
TPMPBMC <-  TPM[ ,which(pheno$group == "PBMCs")]
colnames(TPMPBMC) <- sub("_PBMC", "",pheno$Sample_Name[which(pheno$group == "PBMCs")])
TPMPBMC$DZQV <- NULL   # remove DZQV (Weili)
#save(TPMPBMC, file="TPMPBMC.RData")

########### START from here ############
# define the grouping
allGR0 <- unique(as.character(pheno$group))
GR0 <- allGR0[which(!allGR0  %in% "PBMCs")]
# GR1 <-  c("CD8 Naive","CD8 Memory", "CD4 Naive", "CD4 Memory", "MAIT", "gd Vd2", "gd non Vd2" ,  "Progenitors",  "B Naive", "B Memory", "Plasmablasts", "Mono C", "Mono I" ,"Mono NC" , "NK",  "mDCs","pDCs","Neutrophils" , "Basophils")
#GR1 <-  c("CD4 Naive","CD8 Naive","CD8 Memory", "CD4 Memory", "MAIT", "gd Vd2", "gd non Vd2" ,  "Progenitors",  "B Naive", "B Memory", "Plasmablasts", "Mono NC+I" ,"Mono C" , "NK",  "mDCs","pDCs","Neutrophils" , "Basophils")
GR1 <-  c("T CD4 Naive","T CD8 Naive","T CD8 Memory", "T CD4 Memory", "MAIT", "T gd Vd2", "T gd non-Vd2" ,  "Progenitors",  "B Naive", "B Memory", "Plasmablasts", "Monocytes NC+I" ,"Monocytes C" , "NK",  "mDCs","pDCs","Neutrophils LD" , "Basophils LD")
#GR1 <- c( "B Ex", "Basophils")
# "T cells Naive" "Mono C", "Mono I" ,"Mono NC" 
# get PMBCs expression data
GR0 <- names(sort(colMeans(flowper[colnames(TPMPBMC), GR0]),decreasing=T)) # order by proportions
pheno$group <- as.character(pheno$group)
for(i in 1:length(GR0)){
  names(GR0)[i] <- GR1[GR1 %in% pheno[which(pheno$group == GR0[i])[1],]]
}

GR1 <- unique(names(GR0))
flowtemp <- flowper[colnames(TPMPBMC), GR1]
DEFDR1 <- DEFDR[,c("gene_name", GR1)]
DEFC1 <- DEFC[,c("gene_name", GR1)]

DEFDRmelt <- melt(DEFDR1, id.vars = c("gene_name"))
DEFCmelt <- melt(DEFC1, id.vars = c("gene_name"))

### save flowtemp
save(flowtemp, file="flowtemp.RData")



### FILTER 1 (FOLD CHANGE 2 > and FDR < 0.05)
# arrange them and filter the positive fold change and positive FDR with value less 0.05
FC1 <- DEFCmelt  %>% filter(value > 2 ) %>% arrange(dplyr:::desc(value)) # using the fold change of 2 seems to improve the results respect to smaller fold changes
FDR1 <- DEFDRmelt  %>% filter(value >= 0 & value < 0.05 ) %>% arrange(value) # if you change the threshowld with 0.2 you get worse results

# remove duplicate genes with lower fold change in both FC1 and FDR1
FC1 <- FC1[!duplicated(FC1$gene_name),]
combFC1 <- paste(FC1$gene_name, FC1$variable)
combFDR1 <- paste(FDR1$gene_name, FDR1$variable)
FDR1 <- FDR1[combFDR1 %in% combFC1, ]
# table(FDR1$variable)
MEDg1 <- MEDg[FDR1$gene_name,  GR1]
dim(MEDg1)

datNOfilt <- FDR1 %>% group_by(variable) # %>% slice(1:200) 

### FILTER 2 (LOW EXPRESSION SEQ DEEP)
# filter our those genes that have low expression compared to the max percentage found for that cell type - this filtering improves the results only a bit (significantly less than the previous one), vecause it removes less genes
perMAX <- ceiling(apply(flowtemp, 2, max))/100 # max percentage of each cell type
keep2 <- c()
for(i in colnames(MEDg1)){
  genestemp <- subset(FDR1, variable == i)$gene_name   # genes of each cell type
  idx<-which(MEDg1[genestemp,i] > 1/perMAX[i])  # genes that passed the filtering
  keep2 <- c(keep2, genestemp[idx])
}
MEDg2 <- MEDg1[keep2,]
dim(MEDg2)

### FILTER 3  (HIGH EXPRESSION)
# filter out those genes with very high expression 
filt3 <- apply(MEDg2, 1, function(x) !any(x > 5000))  # this filtering improves the estimation quite a lot too
MEDg3 <- MEDg2[filt3,]
dim(MEDg3)

### FILTER 4  (LOW EXPRESSION)
# filter out those genes with very low expression in all cell types
filt4 <- rowSums(MEDg3) > 50   # this also gives better values (it probably also reduce multicollinearity as it eliminates those genes that have a value of 0 in different cell types)
MEDg4 <- MEDg3[filt4,]
dim(MEDg4)


##### FILTER 5 (DISCRIMINATORY ABILITY)
MEDgtemp <- t(apply(MEDg4, 1, sort, decreasing =T))
filt5 <- (log2(MEDgtemp[,1]) - log2(MEDgtemp[,2])) > 0.1
MEDg5 <- MEDg4[filt5, ]
dim(MEDg5)

##### FILTER 6 (MANUAL FILTER)
# remove the genes that you empirically found to distrupt the deconvolution
BadGenes <- c("VCAN-AS1","SPON2","NELL2","IL23R", "CYP1B1", "HLA-DQB2","BLK","CD19","IGHV3-72", "IGHGP", "IGHA2", "RGS1","CD1C","CST3","HBA1", "CCL5","EIF4A3", "VCAN","HLA-DQA1","CD24","PATL2","KLRK1","TCTEX1D1", "IGHD","NRG1","KLRC1","ANXA2", "MYOM2","HLA-DRB5","IRF7","TRGC1","IL6ST", "IGHG4")
BadGenes2 <- BadGenes[BadGenes %in% rownames(MEDg5)]
MEDg6 <- MEDg5[!rownames(MEDg5) %in% BadGenes2, ] # genes to remove 
dim(MEDg6)
MEDg6 <- MEDg5


AddTheseGenes <- c(
  "FHIT", # important for CD4 Memory
  "CD8A", #important for T cells
  "KLRG1", # improve distinction between CD4 Memory and CD4 Naive
  "NOSIP","LILRA4",  "TRDV2","TRDV1" , "ALPL","TCL1A", "TRBV7-2", "TCN1") #  genes for CD4Naive, pDCs, VD2, non VD2, Basophils, Neutrophils, Basophils B Naive, CD4 Memory

#### FINAL SUBSETTING
FDR6 <- FDR1[FDR1$gene_name %in% rownames(MEDg6), ]

####### subset dataset with FDR values
datfilt <- FDR6 %>% group_by(variable) # %>% slice(1:200) 
# subset(dat, variable=="mDCs")
save(datfilt,datNOfilt, file="datFDRfilt.RData")


#### OPTIMIZATION
# MEDagg <- MEDg[datfilt$gene_name ,GR0]
# MEDagg2 <- t(aggregate(t(MEDagg), by= list(names(GR0)), median)) # aggregate columns
# colnames(MEDagg2)<- MEDagg2[1,]; MEDagg2 <- as.matrix(MEDagg2[-1,])
# class(MEDagg2) <- "numeric"
# 
# rmse <- function(v1, v2) sqrt((mean((v1 - v2)^2)))
# fpp <- function(x){
#   tempCTS <- t(t(SigMat1[, ct, drop=F]) * x)
#   SigMat2 <- SigMat1
#   SigMat2[,ct] <- tempCTS
#   DecLLS <- LLS(SigMat2, MixMat1, nneg = "none", rescale="none")*100
#   errRMSE<- rmse(DecLLS[,ct], flowtemp[,ct])
#   errRMSE
# }
# optFull <- c()
# corFull <- c()
# range <- 30:50
# for(top in range){
#   # top<-30
#   # create the signature and mixed matrix
#   dat2 <- datfilt %>% group_by(variable) %>% dplyr:::slice(1:top)
#   genes <- unique(c(AddTheseGenes,as.character(dat2$gene_name)))
#   SigMat1 <- MEDagg2[genes ,]
#   MixMat1 <- TPMPBMC[rownames(SigMat1),]
# 
#   optRes <- c()
#     for(ct in GR1){
#         res <- optimize( fpp , c(0.1, 30))
#         optRes[ct] <- res$minimum
#    }
#   optFull <- rbind(optFull, optRes)
#   DecLLS <- LLS(SigMat1, MixMat1, nneg = "none", rescale="none")*100
#   corLLS <- diag(cor(flowtemp[,colnames(DecLLS)],DecLLS))
#   corFull <- rbind(corFull, corLLS)
#   print(top)
# }
# colnames(optFull) <- names(optRes)
# rownames(corFull) <-range
# rownames(optFull) <-range
# optRes <- apply(optFull, 2, median, na.rm = T)
# optRes
# 
# # optFull2[which(corFull2< 0.5)] <- NA # select only values with high correlation
# #optRes <- apply(optFull, 2, mean, na.rm = T)
# #optRes
# # naNames <- names(which(is.na(optRes)))
# naNames<- "Progenitors"
# optRes[naNames] <- pheno$pg_median[match(naNames, pheno$group)] * 1.975
# select values according to correlation results
# save(optRes,optFull,corFull,GR0,file="optimizationSigMat.RData")

 
 load( file="optimizationSigMat.RData")
 # prepare the expression matrix for the signature matrix
 MEDg1b <- sweep(MEDg[, names(optRes)], MARGIN=2, optRes, `*`)

 
 ###########################################################
##### DEFINE SIGNATURE MATRIX
# find the matrix with the lowest condition matrix
 dat <- datfilt
k1 <- c()
for(top in 1:100){
  dat2 <- dat %>% group_by(variable) %>% slice(1:top)
  selGenes <- unique(as.character(dat2$gene_name))
  k1 <- c(k1, kappa(MEDg1b[selGenes,colnames(flowtemp)], exact= T))
  if(k1[length(k1)] == min(k1)){ genes <- selGenes}
}
round(min(k1),2)
pdf(paste0("deconOpt/kappa_",namenorm,".pdf"))
par(mar=c(5,4,1,1))
plot( 1:100, k1, ylim=c(0,350), ylab="kappa",xlab="number of genes",xaxt="n", pch = 20) # main=paste0("min = ", round(min(k1),2))
val <- seq(0, 100*length(GR1), by = length(GR1))
axis(1, at = seq(1,100,by=5), labels=val[seq(1,100,by=5)],las=2)
dev.off()

# choose it manually
 dat2 <- dat %>% group_by(variable) %>% dplyr:::slice(1:40)
 genes <- unique(as.character(dat2$gene_name))

# if you want to make sure that good discriminatory genes are at the top of the list:
genes <- unique(c(AddTheseGenes,genes)) 
write.table(MEDg1b[unique(as.character(dat$gene_name)), GR1], "deconOpt/sigmatrixRNAseqFULL.txt", sep="\t" )

SigMat1 <- MEDg1b[genes, GR1]
MixMat1 <- TPMPBMC[rownames(SigMat1),]
write.table(SigMat1, "deconOpt/sigmatrixRNAseq.txt", sep="\t" )

####  DECONVOLUTION!!!
#Dec <- CIBERSORT(sig_matrix = SigMat1, mixture_file = MixMat1,QN=F, absolute=F, abs_method = 'no.sumto1')*100  #  absolute=T, abs_method = 'no.sumto1'
Dec <- LLS(SigMat1, MixMat1, nneg = "none", rescale="none")*100 # you can set nneg to "iterate" or "pmax"
#Dec <- t((apply(MixMat1, 2, function(x) coef(rlm( as.matrix(SigMat1), x )))) *100 )
cccDec <- c()
 for(name in colnames(Dec)) cccDec[name] <- unlist(epi.ccc(Dec[,name], flowtemp[,name])$rho.c[1])
 corDec <- diag(cor(flowtemp[,colnames(Dec)],Dec))
# round(ccc["Mono C"],3)


#### PICTURES!
colors <- brewer.pal(9, "Set1")
pal <- colorRampPalette(colors)
labelColors <- pal(length(colnames(Dec)))
names(labelColors) <- colnames(Dec)

for(name in colnames(Dec)){
  DecCT <- Dec[,name]
  FACS <- flowtemp[,name]
  cccD <- round(cccDec[name],3)
  corD <- round(corDec[name],3)
  scatplot <-  qplot(x=FACS, y= DecCT, size=I(0.8)) +
              theme_bw() + theme(panel.grid.major = element_blank(), panel.grid.minor = element_blank(), axis.text=element_text(size=6), 
              panel.border = element_blank(), axis.line = element_line(colour="black", size=0.2), 
              axis.title = element_blank(), plot.title = element_text( face = "bold", size=8)) +  #color=labelColors[name],
               ggtitle(name) +  xlim(0, max(FACS)+0.2) + ylim(0, max(DecCT)) + stat_smooth(method=lm, se=FALSE, colour="darkolivegreen4")  +
               annotate("text", x=max(FACS)*0.01, y=max(DecCT), label=paste0("ccc = ",cccD), size =2,hjust=0) + 
               annotate("text", x=max(FACS)*0.01, y=max(DecCT)*0.90, label=paste0("r = ",corD), size =2,hjust=0) # + geom_text(aes(label=rownames(DecLLS)),hjust=0, vjust=0,size=2)
  # scatplot
  ggsave(paste0("deconOpt/", sub("/","-" ,name), ".pdf"),scatplot, width = 1.3, height = 1.3,units="in")
}


# ## define IQR for each gene in each cell type - I tried using also the logarithm of TPM values, but it squeeze the high values too much.
# GRdf <- cbind.data.frame(as.character(pheno$group), pheno[,grep("group[0-9]", colnames(pheno))], stringsAsFactors =F)
# GRvec <- unique(unlist(GRdf))
# IQRg <-c()
# for(i in GRvec){
#   idGR<- which(apply(GRdf, 1, function(r) any(r %in% i)))
#   IQRg <- cbind(IQRg, apply(TPM[,idGR], 1, IQR))
# }
# colnames(IQRg) <- GRvec
# rownames(IQRg) <- DEFDR$gene_name
# save(IQRg, file="IQRg.RData")
# load("IQRg.RData")
# IQRg1 <- IQRg[,GR1]
## test single genes
#idGR<- which(apply(GRdf, 1, function(r) any(r %in% "B Naive")))
#TPM["TCL1A",idGR]


# # create FC and FDR combination of genes with FDR lower than 0.2
# DEFDRFC <- as.matrix(DEFDR[,GR1])
# DEFDRFC[] <- 0
# for(i in GR1){
#   genes <- DEFDR[,i] <0.2
#   DEFDRFC[genes,i] <- abs(log10(DEFDR[genes,i]))*DEFC[genes,i]
# }
# DEFDRFC[!is.finite(DEFDRFC)] <- 0


# # melt it
# DE <- as.data.frame(DEFDRFC)
# DE$geneID <- rownames(DEFDRFC)
# DEmelt <- melt(DE, id.vars = c("geneID"))
# DEmelt <- DEmelt %>% filter(value > 0 ) %>% arrange(desc(value))
# #remove duplicates
# DEmelt <- DEmelt[!duplicated(DEmelt$geneID),]

# in case you want to check genes that are overexpressed in one sample
# SUMg <- MEDg2
# SUMg[] <- 0
# for(i in 1:ncol(MEDg2)){
#   id<- which(DEFDR2[,i] < 0.2)
#   sumtemp <- apply(MEDg2[id,-i], 1, sum)
#   SUMg[id, i] <- MEDg2[id,i] - sumtemp
# }
# SUMg$geneID <- rownames(SUMg) 
# SUMgm <- melt(SUMg, id.vars = c("geneID"))
# SUMdat <- SUMgm %>% arrange(desc(value))
# aa <- subset(SUMdat, variable=="CD4 Naive")

### remove 0 values and sort it
#dat <- DEmelt %>% filter(value > 2 ) %>% arrange(desc(value)) %>% group_by(variable) %>% slice(1:15)
### apply empirical method
# dattemp <- dat %>% group_by(variable) %>% slice(1)
# genesV <- dattemp$geneID
# names(genesV) <- colnames(MEDg)
# corV <- rep(0, ncol(MEDg))
# names(corV) <- colnames(MEDg)
# count <-1
# count2 <- 0
# while(count != 0){
#   count <- 0
#   for(i in colnames(MEDg)){
#     genesTemp <- genesV[colnames(MEDg) != i]
#     dat2 <- dat%>% filter(variable == i)
#     for(a in 1:nrow(dat2)){
#       gTemp <- dat2 %>% slice(a)
#       genesTdec <- unlist(c(genesTemp, gTemp$geneID))
#       DecLLS <- LLS(MEDg2[genesTdec, ] , TPMPBMC[genesTdec, ], nneg = "iterate", rescale="none")*100
#       corLLS <- diag(cor(flowtemp[,colnames(DecLLS)],DecLLS))
#       if(is.na(corLLS[i]) ) corLLS[i] <-0
#       if(corLLS[i] > corV[i]) {
#         corV[i] <- corLLS[i]
#         genesV[i] <- as.character(gTemp)
#         count = count+1
#       }
#     }
#   }
#   print(count2)
#   count2 <- count2+1
# }
# save(genesV, file="genesVminDEC2.RData")
# load("genesVminDEC.RData")

# FILTERING ATTEMPTS
# filter out genes that are expressed in only one cell type to avoid multicollinearity and to take those genes with higher signal in general
#filter2 <- apply(MEDg2, 1, function(x) length(x[x>0])>=ceiling(ncol(MEDg2)/5)) # filter out genes that have 0 expression in more than 1/5 of the samples
#MEDg2 <- MEDg2[filter2,]


# other optimization selection methods
# corFull2 <- corFull[,names(GR0)]
# colnames(corFull2) <- GR0
# optFull2 <- optFull[,GR0]
# optFull2[which(corFull2< 0.5)] <- NA # select only values with high correlation
# optRes2 <- apply(optFull2, 2, median, na.rm = T)
# optRes2
# 
# optFull4 <- optFull3[which(apply(corFull3, 1, function(x) all(x > 0.5))),]
# optRes3 <- apply(optFull4, 2, median, na.rm = T)
# optRes3
# 
# pgmed0 <- pg_med$x
# names(pgmed0) <- pg_med$Group.1
# aa<- apply(optFull, 1, IQR)
# 
# # substitute NA with median RNA yield
# naNames <- names(which(is.na(optRes)))
# optRes[naNames] <- pheno$pg_median[match(naNames, pheno$group)] * 1.975


# some genes manually retrieved!
# genesV2 <- c("CD248" , "CD8A","NOG" ,"TNFRSF4","SLC4A10","TRDV2","TRDV1" ,"ERG", "TCL1A","COL4A3" ,"IGHV3-63" ,"S1PR3" ,"C1QA" , "AC104809.4" , "TRDJ1", "CD1E", "LILRA4" , "ALPL", "RP11-13A1.3" ) # this is manually adjusted and it seems to work well! Keep it for a while and then you see.
# names(genesV2) <- c("CD8 Naive","CD8 Memory","CD4 Naive","CD4 Memory","MAIT", "gd Vd2" ,"gd non Vd2" ,"Progenitors" ,"B Naive" , "B Memory","Plasmablasts","Mono C","Mono I","Mono NC","NK" , "mDCs" ,"pDCs" ,"Neutrophils","Basophils" )


## MULTIPLE OPTIMIZATION
#### OPTIMIZATION
# define the lower bound according to the median and sd of RNA yield
# pg_sd <- aggregate(pheno$pg_cell* 1.975,by=list(pheno$group), sd) # sd rna yield
# pg_med <- aggregate(pheno$pg_cell* 1.975,by=list(pheno$group), median) # median rna yield
# 
# lowB <- pg_med$x - (2*pg_sd$x)
# names(lowB) <- pg_sd$Group.1
# lowB <- pmax(lowB,0.1)   # lower bound
# 
# upB <- pg_med$x + (2*pg_sd$x)
# names(upB) <- pg_sd$Group.1
# upB <- pmax(upB,1)   # upper bound
# increase <- which(pg_sd$Group.1 %in% c("pDCs","mDCs", "Plasmablasts","Mono C", "Mono NC", "Mono I"))
# upB[increase]<- pg_med$x[increase] + (8*pg_sd$x[increase])
# 
# rmse <- function(v1, v2) sqrt((mean((v1 - v2)^2)))
# fpp <- function(x){
#   tempCTS <- t(t(SigMat1[, cts, drop=F]) * x)
#   SigMat2[,ct] <- apply(tempCTS[,,drop=F], 1, median)   # summarize and replace the new cell type
#   DecLLS <- LLS(SigMat2, MixMat1, nneg = "none", rescale="none")*100
#   errRMSE<- rmse(DecLLS[,ct], flowtemp[,ct])
#   errRMSE
# }
# optFull <- c()
# corFull <- c()
# for(top in 10:40){
#   # top<-10
#   # create the signature and mixed matrix
#   dat2 <- datfilt %>% group_by(variable) %>% slice(1:top)
#   genes <- unique(as.character(dat2$gene_name))
#   SigMat1 <- MEDg[genes ,GR0]
#   MixMat1 <- TPMPBMC[rownames(SigMat1),]
#   SigMat2 <- t(aggregate(t(SigMat1), by= list(names(GR0)), median)) # aggregate columns
#   colnames(SigMat2)<- SigMat2[1,]; SigMat2 <- as.matrix(SigMat2[-1,])
#   class(SigMat2) <- "numeric"
#   
#   optRes <- c()
#   for(ct in GR1){
#     cts <- GR0[names(GR0) ==ct]
#     if(length(cts) == 1){ # optimize the single cell type
#       res <- optimize( fpp , c(lowB[cts], upB[cts]))
#       optRes[cts] <- res$minimum
#     }else{  # optimize multiple cell types
#       res <- optim(par=rep(1, length(cts)), fn=fpp, lower=lowB[cts], upper=upB[cts], method="L-BFGS-B")
#       optRes[cts] <- res$par
#     }
#     #SigMat2[,ct] <- apply(t(t(SigMat1[, cts, drop=F]) * optRes[cts]), 1, median)   # summarize the new cell type - comment it out if you want to keep the original TPM values for the optimization
#   }
#   optFull <- rbind(optFull, optRes)
#   colnames(optFull) <- names(optRes)
#   DecLLS <- LLS(SigMat2, MixMat1, nneg = "none", rescale="none")*100
#   corLLS <- diag(cor(flowtemp[,colnames(DecLLS)],DecLLS))
#   corFull <- rbind(corFull, corLLS)
#   print(top)
# }
# rownames(corFull) <-10:40
# rownames(optFull) <-10:40
# optRes <- apply(optFull, 2, median, na.rm = T)
# optRes
# 
# corFull2 <- corFull[,names(GR0)]
# colnames(corFull2) <- GR0
# optFull2 <- optFull[,GR0]
# optFull2[which(corFull2< 0.5)] <- NA # select only values with high correlation
# optRes <- apply(optFull2, 2, median, na.rm = T)
# optRes
# naNames <- names(which(is.na(optRes)))
# optRes[naNames] <- pheno$pg_median[match(naNames, pheno$group)] * 1.975
# # select values according to correlation results
# save(optRes,optFull,corFull,optFull2, GR0,file="optimizationSigMat.RData")


##### LINEAR MODELING mRNA yield calculation 2
# define the lower bound according to the median and sd of RNA yield
# optFull <- c()
# corFull <- c()
# #for(top in 10:40){
#   top<-30
#   #for(ind in 1:12){ 
#   ind <- 11
#   # create the signature and mixed matrix
#   dat2 <- datfilt %>% group_by(variable) %>% slice(1:top)
#   genes <- unique(as.character(dat2$gene_name))
#   SigMat1 <- MEDg[genes ,GR0]
#   MixMat1 <- TPMPBMC[rownames(SigMat1),]
#   SigMat2 <- t(aggregate(t(SigMat1), by= list(names(GR0)), median)) # aggregate columns
#   colnames(SigMat2)<- SigMat2[1,]; SigMat2 <- as.matrix(SigMat2[-1,])
#   class(SigMat2) <- "numeric"
#   SigMat3 <- sweep(SigMat2 , MARGIN=2, as.numeric(flowtemp[ind,colnames(SigMat2)]), `*`)
#   
#   DecLLS <- LLSsingle(SigMat3, MixMat1[,ind,drop=F], nneg = "none", rescale="none")*100
#   DecLLS
