SaveName <- "kal_genes" # assign a value between STAR, kal_tx and kal_genes
source("~/Dropbox/_Deconvolution_paper/Src/4.RNAseqObj.R") 
source("~/Desktop/Work_temp/Source code/deconvolution/LLS.R")
source("~/Desktop/Work_temp/Source code/deconvolution/CIBERSORT.R")

# load the libraries
libraries <- "dplyr,reshape2,edgeR,ggplot2,RColorBrewer,e1071,preprocessCore,epiR"
libraries <- strsplit(libraries, ",")[[1]]
for (name in libraries) {
  suppressPackageStartupMessages(library(name, character.only=T, warn.conflicts=F, quietly=T, verbose=F))
}
load(file= "../../CellPercentageFULL.RData")
load("TPM_geneName.RData") # TPM
load("DE_geneName.RData") # DEFDR and DEFC
load("MedianGeneName.RData") # MEDg
load("TPMPBMC.RData")  # TPMPBMC
load("optimizationSigMat.RData") # optRes
load("../../Microarrays/Comb_TN.RData")
dir.create("deconMICRO", showWarnings = FALSE)

# aggregate probes to gene symbols
CTNsub2 <- CTNsub[which(fData(CTNsub)$SYMBOL %in% DEFDR$gene_name), ]
micPBMC0 <-aggregate(exprs(CTNsub2),by=list(fData(CTNsub2)$SYMBOL), FUN = max)
rownames(micPBMC0) <- micPBMC0$Group.1
micPBMC0$Group.1 <- NULL
micPBMC0$DZQV <- NULL
micPBMC <- 2^micPBMC0
# idPBMC <- colnames(micPBMC) %in% rownames(flowper)
TPMPBMC <- TPMPBMC[rownames(micPBMC),]
MEDg <- MEDg[rownames(micPBMC),]
#plot(log2(TPMPBMC[,"9JD4"]+1), MicPBMCnorm[,"9JD4"]-4)

# CORRELATION OF THE GENES 
corre <-c()
for(i in 1:nrow(TPMPBMC)){
  corre<- c(corre,cor(log2(as.numeric(TPMPBMC[i,])+1), log2(micPBMC[i, colnames(TPMPBMC)]+1)))
}
corre[is.na(corre)] <- 0
names(corre) <- rownames(TPMPBMC)
corTHR <- 0.7 # define here the threshold for the correlating genes
table(corre>corTHR)
UQmic <- quantile(as.matrix(micPBMC[corre>corTHR,]), 0.75)
UQtpm <- quantile(as.matrix(TPMPBMC[corre>corTHR,]), 0.75)
sc <-  UQmic / UQtpm # this is my scaling factor
sc

gene<- "KCNG1"
#plot(log2(TPMPBMC[gene,]+1), log2(micPBMC[gene,colnames(TPMPBMC)]+1))
plot(as.numeric(TPMPBMC[gene,]), micPBMC[gene,colnames(TPMPBMC)])
cor(as.numeric(TPMPBMC[gene,]), micPBMC[gene,colnames(TPMPBMC)])


# NORMALIZATION
micPBMCn <- micPBMC / sc
save(micPBMCn, file="../../Microarrays/micPBMCnormalized.RData")
# I don't need to find the target quintile anymore. I just know that the maximum microarray value should be 25000
# targetQN <- normalize.quantiles.determine.target(as.matrix(micPBMCn[,1]))
# write.table(as.matrix(targetQN), "deconMICRO/TargetQuantile.txt", sep="\t", row.names=rownames(micPBMCn),col.names = F)

########### START from here ############
# define the grouping
#allGR0 <- unique(as.character(pheno$group))
#GR0 <- allGR0[which(!allGR0  %in% "PBMCs")]
GR0 <- names(optRes)
# GR1 <-  c("CD8 Naive","CD8 Memory", "CD4 Naive", "CD4 Memory", "MAIT", "gd Vd2", "gd non Vd2" ,  "Progenitors",  "B Naive", "B Memory", "Plasmablasts", "Mono C", "Mono I" ,"Mono NC" , "NK",  "mDCs","pDCs","Neutrophils" , "Basophils")
#GR1 <-  c("T cells Naive","CD8 Memory", "CD4 Memory", "MAIT", "gd Vd2", "gd non Vd2" ,  "Progenitors",  "B Naive", "B Memory", "Plasmablasts", "Mono NC+I" ,"Mono C" , "NK",  "mDCs","pDCs","Neutrophils" , "Basophils")
#GR1 <-  c("CD4 Naive","CD8 Naive","CD8 Memory", "CD4 Memory", "MAIT", "gd Vd2", "gd non Vd2" ,  "Progenitors",  "B Naive", "B Memory", "Plasmablasts", "Mono NC+I" ,"Mono C" , "NK",  "mDCs","pDCs","Neutrophils" , "Basophils")
GR1 <-  c("T CD4 Naive","T CD8 Naive","T CD8 Memory", "T CD4 Memory",   "B Naive", "B Memory", "Plasmablasts", "NK",  "pDCs","Neutrophils LD" , "Basophils LD", "T gd","MAIT", "Progenitors", "mDCs", "Monocytes")
# GR1 <-  c("T CD4","T CD8", "B Naive", "B Memory", "Plasmablasts", "NK",  "pDCs","Neutrophils LD" , "Basophils LD", "T gd","MAIT", "Progenitors", "mDCs", "Monocytes")

# "Mono NC+I" ,"Mono C" , 
GR1 <- names(sort(colMeans(flowper[colnames(TPMPBMC), GR1]),decreasing=T)) # order by the proportion
pheno$group <- as.character(pheno$group) # make sure that pheno$group is a character vector
for(i in 1:length(GR0)){   # put names on of broader categories to single cell types
  names(GR0)[i] <- GR1[GR1 %in% pheno[which(apply(pheno, 1, function(x) any(x %in% GR0[i])))[1],]]
}


flowtemp <- flowper[colnames(micPBMCn), GR1]
DEFDR1 <- DEFDR[DEFDR$gene_name %in% rownames(micPBMCn),c("gene_name", GR1)]
DEFC1 <- DEFC[DEFC$gene_name %in% rownames(micPBMCn),c("gene_name", GR1)]
mFDR <- melt(DEFDR1, id.vars = c("gene_name"))
mFC <- melt(DEFC1, id.vars = c("gene_name"))
MEDg1 <- MEDg[rownames(micPBMCn),GR1]


save(mFC, file="deconMICRO/FC_micro.RData")
save(mFDR, file="deconMICRO/FDR_micro.RData")

### FILTER 1 (FOLD CHANGE 0.2 > and FDR < 0.05)
# arrange them and filter the positive fold change and positive FDR with value less 0.05
mFC1 <- mFC  %>% filter(value > 2 ) %>% arrange(desc(value)) # using the fold change of 2 seems to improve the results respect to smaller fold changes
mFDR1 <- mFDR  %>% filter(value > 0 & value < 0.05 ) %>% arrange(value) # if you change the threshowld with 0.2 you get worse results

# remove duplicate genes with lower fold change in both FC1 and FDR1
mFC1 <- mFC1[!duplicated(mFC1$gene_name),]
combFC1 <- paste(mFC1$gene_name, mFC1$variable)
combFDR1 <- paste(mFDR1$gene_name, mFDR1$variable)
mFDR1 <- mFDR1[combFDR1 %in% combFC1, ]
# table(FDR1$variable)
MEDg1 <- MEDg[mFDR1$gene_name,  GR1]
dim(MEDg1)


### FILTER 2 (LOW EXPRESSION in microarrays)   OFF
# filter out those genes with very high expression 
filt2 <- apply(micPBMC[rownames(MEDg1), ], 1, function(x) !any(x < 0))  # put 0 if you want to turn it off
MEDg2 <- MEDg1[filt2, ]
dim(MEDg2)

### FILTER 3 (HIGH EXPRESSION in microarrays)   OFF
# filter out those genes with very high expression 
filt3 <- apply(micPBMC[rownames(MEDg2), ], 1, function(x) !any(x > 100000))   # put a value larger than 100000 if you want to turn it off
MEDg3 <- MEDg2[filt3, ]
dim(MEDg3)

# FILTER 4 (based on correlation) 
MEDg4 <- MEDg3[corre[rownames(MEDg3)] > -0.4, ]
dim(MEDg4)


### FILTER 5  (HIGH EXPRESSION SIG) 
# filter out those genes with very high expression 
filt5 <- apply(MEDg4, 1, function(x) !any(x > 5000))  # this filtering improves the estimation quite a lot too (especially for B cell memory)
MEDg5 <- MEDg4[filt5,]
dim(MEDg5)

### FILTER 6  (LOW EXPRESSION)
# filter out those genes with very low expression in all cell types
filt6 <- rowSums(MEDg5) > 10   # this also gives better values (it probably also reduce multicollinearity as it eliminates those genes that have a value of 0 in different cell types)
MEDg6 <- MEDg5[filt6,]
dim(MEDg6)


### FILTER 7  (MINIMUM EXPRESSION) - OFF
# filter out those genes that do not reach a minimum expression in no cell type
filt7a <- apply(MEDg6, 1, function(x) any(x > 0))    # a cell type has to have at least the value decided here
MEDg7a <- MEDg6[filt7a,]
dim(MEDg7a)

### FILTER 7  (sd) - OFF
# filter out those genes that do not reach low sd
filt7b <- apply(MEDg7a, 1, function(x) sd(x) >= 5)    # a cell type has to have at least the value decided here
MEDg7 <- MEDg7a[filt7b,]
dim(MEDg7)

##### FILTER 8 (DISCRIMINATORY ABILITY) # this improve a bit everything
MEDgtemp <- t(apply(MEDg7, 1, sort, decreasing =T))
filt8 <- (log2(MEDgtemp[,1]) - log2(MEDgtemp[,2])) > 0.2
MEDg8 <- MEDg7[filt8, ]
dim(MEDg8)

##### FILTER 9 (MANUAL FILTER)
# remove the genes that you empirically found to distrupt the deconvolution
MEDg9 <- MEDg8[!rownames(MEDg8) %in% c("WARS", "ALAS2","IL7R","HSP90B1","CD28","PRCP","FAM46C", "RBP7","CYP1B1", "HLA-DQB2","BLK","CD19","IGHV3-72", "IGHGP", "IGHA2", "RGS1","CD1C","CST3","HBA1", "CCL5","EIF4A3", "VCAN","HLA-DQA1","CD24","PATL2","KLRK1","TCTEX1D1", "IGHD","NRG1","KLRC1","ANXA2", "MYOM2","HLA-DRB5","IRF7","TRGC1","IL6ST","GZMA"), ] # genes to remove 
dim(MEDg9)

AddTheseGenes <- c("KLRD1","IL18RAP","KLRD1","CD33","B4GALT5",  "GSTA4","SOCS2","ZNF821", "CD2", "LGMN","PLD1","SEC61B", "F2RL1","SORT1")
#### FINAL SUBSETTING
dat <- mFDR1[mFDR1$gene_name %in% rownames(MEDg9), ] %>% group_by(variable) # %>% slice(1:200) 
# subset(dat, variable=="mDCs")
save(dat, file="deconMICRO/filtered_microarray.RData")

##### DEFINE SIGNATURE MATRIX
# find the matrix with the lowest condition matrix
k1 <- c()
for(top in 20:80){
  dat2 <- dat %>% group_by(variable) %>% slice(1:top)
  selGenes <- unique(as.character(dat2$gene_name))
  k1 <- c(k1, kappa(MEDg1[selGenes,colnames(flowtemp)], exact= T))
  if(k1[length(k1)] == min(k1)){ genes <- selGenes}
}
#png(paste0("deconv_yield/kappa_",namenorm,".png"))
plot( 20:80, k1, ylim=c(0,150), main=paste0("min = ", round(min(k1),2)))
#dev.off()

###################
#### OPTIMIZATION
## use RNAseq optimization
## MEDg_NORM <- sweep(MEDg[, names(optRes)], MARGIN=2, optRes, `*`)
## MEDg_NORM_AGG <- t(aggregate(t(MEDg_NORM[,GR0]), by= list(names(GR0)), median)) # aggregate columns
## colnames(MEDg_NORM_AGG)<- MEDg_NORM_AGG[1,]; MEDg_NORM_AGG <- as.matrix(MEDg_NORM_AGG[-1,])
## class(MEDg_NORM_AGG) <- "numeric"


# #### Optimize again
# MEDagg <- MEDg[dat$gene_name ,GR1]
# rmse <- function(v1, v2) sqrt((mean((v1 - v2)^2)))
# fpp <- function(x){
#   tempCTS <- t(t(SigMat1[, ct, drop=F]) * x)
#   SigMat2 <- SigMat1
#   SigMat2[,ct] <- tempCTS
#   DecLLS <- LLS(SigMat2, MixMat1, nneg = "none", rescale="none")*100
#   errRMSE<- rmse(DecLLS[,ct], flowtemp[,ct])
#   errRMSE
# }
# optFullM <- c()
# corFullM <- c()
# range<-30:60
# for(top in range){
#   # top<-30
#   # create the signature and mixed matrix
#   dat2 <- dat %>% group_by(variable) %>% slice(1:top)
#   genes <- unique(c(AddTheseGenes, as.character(dat2$gene_name)))
#   SigMat1 <- MEDagg[genes ,]
#   MixMat1 <- micPBMCn[rownames(SigMat1),]
# 
#   optResM <- c()
#     for(ct in GR1){
#         res <- optimize( fpp , c(0.1, 30))
#         optResM[ct] <- res$minimum
#    }
#   optFullM <- rbind(optFullM, optResM)
#   DecLLS <- LLS(SigMat1, MixMat1, nneg = "none", rescale="none")*100
#   corLLS <- diag(cor(flowtemp[,colnames(DecLLS)],DecLLS))
#   corFullM <- rbind(corFullM, corLLS)
#   print(top)
# }
# colnames(optFullM) <- names(optResM)
# rownames(corFullM) <-range
# rownames(optFullM) <-range
# # optResM <- optFullM["45",]
# optFullM[optFullM>29] <-NA
# optResM <- apply(optFullM, 2, median, na.rm = T)
# optResM
# 
# # optFullM2[which(corFullM2< 0.5)] <- NA # select only values with high correlation
# #optResM <- apply(optFullM, 2, mean, na.rm = T)
# #optResM
# # naNames <- names(which(is.na(optRes)))
# naNames<- c("Progenitors")
# optResM[naNames] <- optRes[naNames]
# # select values according to correlation results
# save(optResM,optFullM,corFullM,file="deconMICRO/optimizationSigMatMicro.RData")

load( file="deconMICRO/optimizationSigMatMicro.RData")
# prepare the expression matrix for the signature matrix

MEDg_NORM_AGG <- sweep(MEDg[dat$gene_name, names(optResM)], MARGIN=2, optResM, `*`)

GR2 <- GR1 #[which(GR1!="Progenitors")]

# choose the genes manually
dat2 <- dat %>% group_by(variable) %>% slice(1:40)
genes <- unique(as.character(dat2$gene_name))
# if you want to make sure that good discriminatory genes are at the top of the list:
genes <- unique(c(AddTheseGenes,genes)) # genes for CD8 Mem, Basophils, CD4 Naive, plasmablasts

write.table(MEDg_NORM_AGG[unique(as.character(dat$gene_name)), GR2], "deconMICRO/sigmatrixMicroFULL.txt", sep="\t" )
#GR2<-GR1[!GR1 %in% "T gd"]
SigMat1 <- MEDg_NORM_AGG[genes, GR2]
MixMat1 <- micPBMCn[rownames(SigMat1), ]

write.table(SigMat1, "deconMICRO/sigmatrixMicro.txt", sep="\t" )

####  DECONVOLUTION!!!
# Dec <- CIBERSORT(sig_matrix = SigMat1, mixture_file = MixMat1,QN=F)   #, absolute = T, abs_method = 'no.sumto1'
Dec <- LLS(SigMat1, MixMat1, nneg = "none", rescale="none")*100 # you can set nneg to "iterate" or "pmax"
cccDec <- c()
Dec[Dec<0]=0
for(name in colnames(Dec)) cccDec[name] <- unlist(epi.ccc(Dec[,name], flowtemp[,name])$rho.c[1])
corDec <- diag(cor(flowtemp[,colnames(Dec)],Dec))
# round(ccc["Mono C"],3)


#### PICTURES!
for(name in colnames(Dec)){
  DecCT <- Dec[,name]
  FACS <- flowtemp[,name]
  cccD <- round(cccDec[name],3)
  corD <- round(corDec[name],3)
  scatplot <-  qplot(x=FACS, y= DecCT, size=I(0.8)) +
    theme_bw() + theme(panel.grid.major = element_blank(), panel.grid.minor = element_blank(), axis.text=element_text(size=6), 
                       panel.border = element_blank(), axis.line = element_line(colour="black", size=0.2), 
                       axis.title = element_blank(), plot.title = element_text( face = "bold", size=8)) +  #color=labelColors[name],
    ggtitle(name) +  xlim(0, max(FACS)+0.2) + ylim(0, max(DecCT)) + stat_smooth(method=lm, se=FALSE, colour="darkolivegreen4")  +
    annotate("text", x=max(FACS)*0.01, y=max(DecCT), label=paste0("ccc = ",cccD), size =2,hjust=0) + 
    annotate("text", x=max(FACS)*0.01, y=max(DecCT)*0.90, label=paste0("r = ",corD), size =2,hjust=0) # + geom_text(aes(label=rownames(DecLLS)),hjust=0, vjust=0,size=2)
  # scatplot
  ggsave(paste0("deconMICRO/", sub("/","-" ,name), ".pdf"),scatplot, width = 1.3, height = 1.3,units="in")
}




# you might also want to try not log2 transform the data or not doing the batch effect correction

# quantile normalization
# MixQN <- normalize.quantiles.determine.target(as.matrix(TPMPBMC[rownames(SigMat3), ]))
# MixQN <- apply(TPMPBMC[rownames(SigMat3), ] , 1 ,median) # this was my initial version!
# MicPBMCnorm <- normalize.quantiles.use.target(micPBMC[genes,],target = MixQN) #[datNOfilt$gene_name, ]) ,target = MixQN )
# colnames(MicPBMCnorm) <- colnames(micPBMC)
# rownames(MicPBMCnorm) <- rownames(micPBMC[genes,]) # [datNOfilt$gene_name, ])
