libraries <- strsplit("ggplot2,edgeR,RColorBrewer", ",")[[1]]
for (name in libraries) {
  suppressPackageStartupMessages(library(name, character.only=T, warn.conflicts=F, quietly=T, verbose=F))
}

SaveName <- "kal_genes" # assign a value between STAR, kal_tx and kal_genes
source("~/Dropbox/_Deconvolution_paper/Src/4.RNAseqObj.R") 

load("TPM_geneName.RData")
HK1 <- read.table("~/Desktop/Work_temp/Deconvolution/RNA-seq/genes_info/HK_genes_Eisenberg2013.txt",stringsAsFactors =F) 
HK2 <- read.table("~/Desktop/Work_temp/Deconvolution/RNA-seq/genes_info/HK_genes_Hsiao2011.txt",stringsAsFactors =F) 
HK <- unique(c(HK1[,1],HK2[,1]))

ordTPM <- c("PBMCs","Progenitors","B Naive","B NSM","B Ex","B SM","Plasmablasts","T CD4 Naive","Tfh","Tregs","Th1","Th1/Th17","Th17","Th2","T CD4 EMRA", "T CD8 Naive","T CD8 CM","T CD8 EM","T CD8 EMRA","T gd Vd2","T gd non-Vd2","MAIT","NK", "pDCs","mDCs", "Monocytes C", "Monocytes I", "Monocytes NC", "Neutrophils LD","Basophils LD") # you can also take the first column of the file: grouping_cell_types.txt
idORD <- order(match(pheno$group, ordTPM))
TPMord <- TPM[,idORD] 
namesOrd <- pheno$group[idORD]
genesHK <- HK[HK %in% rownames(TPM)]
genesNonHK <- rownames(TPM)[!rownames(TPM) %in% genesHK]

# save normFactor HK
sc1 <- apply(TPM[genesHK, ],2, median)
sc2 <- (1/sc1)*37
save(sc2, file="HKfact.RData")

#check HK with tpm values
tpmhk <- TPMord[genesHK, ]
boxplot(log2(tpmhk+1), names=namesOrd, las=2)
# boxplot(log(TPMord+1), names=namesOrd, las=2)

#check HK normalized by median values
sc <- apply(tpmhk,2, median)
TPM_HK <- sweep(TPMord, MARGIN=2, STATS=(1/sc)*37,FUN="*")
tpmhk2 <- TPM_HK[genesHK, ]
boxplot(log(tpmhk2+1), names=namesOrd, las=2)
# boxplot(log(TPM_HK+1), names=namesOrd, las=2)


#check HK normalized with TMM
TMM_fact <- calcNormFactors(TPMord, refColumn = 1)
tmm_norm <- sum(TPMord[,1]) / (colSums(TPMord) * TMM_fact )
TPM_TMM <- sweep(TPMord, MARGIN=2, STATS=tmm_norm, FUN="*")
tpmhk3 <- TPM_TMM[genesHK, ]
boxplot(log(tpmhk3+1), names=namesOrd, las=2)
boxplot(log(TPM_TMM+1), names=namesOrd, las=2)
cor(tmm_norm, 1/sc) # correlation tmm and scaling factor HK


## Check housekeeping with deconvolution normalization
load(file="optimizationSigMat.RData")
optRes2 <- optRes[names(GR0)]
names(optRes2) <- GR0
optRes3 <- c(1,optRes2)
names(optRes3)[1] <- "PBMCs"
TPM_DEC <- sweep(TPMord, MARGIN=2, optRes3[as.character(namesOrd)], `*`)
tpmhk4 <- TPM_DEC[genesHK, ]
boxplot(log2(tpmhk4+1), names=namesOrd, las=2)
boxplot(log2(TPM_DEC+1), names=namesOrd, las=2)


## Check housekeeping with cell yield normalization
TPM_C <- sweep(TPMord, MARGIN=2, pheno$pg_cell[idORD]*1.95, `*`)
tpmhk5 <- TPM_C[genesHK, ]
boxplot(log(tpmhk5+1), names=namesOrd, las=2, ylim=c(0,15))


### NOW LET'S EXPLORE THE HK normalized by their median value
#filter
namenorm <- dir(pattern="tpm")[6]  
TPMnorm <- read.table(paste0(namenorm, "/tpm_norm.txt"), sep="\t", header=T, row.names = 1,check.names = FALSE)
de <- read.delim(paste0(namenorm, "/DEdecFDR.txt"), header = T,check.names = F, row.names = 1) 
#TPMnorm <- TPM_TMM  # remember to change also this together with the namenorm
#rownames(de) <- rownames(TPM)
TPMnorm <- TPMnorm[rownames(de), ]
gn <- as.character(de$gene_name)
de$gene_name <- NULL
ei <- de$entrez_id
de$entrez_id <- NULL
ct <- colnames(de)
meanT <- apply(log2(TPMnorm+1), 1, mean)
#geneH <- names(which(meanT>0))
var <- apply(log2(TPMnorm+1), 1, sd)
boxplot(var[gn %in% genesHK],var[gn %in% genesNonHK], names=c("HK genes", "non-HK genes"), ylab="standard deviation")

upfin <-c()
downfin <-c()
for(i in 1:nrow(de)){
  up <- which(de[i,] < 0.05 & de[i,] > 0)
  down <- which(de[i,] < 0 & de[i,] > -0.05)
  upfin<- c(upfin, paste(ct[up], collapse = ", "))
  downfin<- c(downfin, paste(ct[down], collapse = ", "))
}

res <- cbind(gn, ei, round(meanT,3), round(var,3), upfin, downfin, gn %in% HK1[,1], gn %in% HK2[,1])
colnames(res) <- c("Gene name", "ENTREZ ID", "mean expression", "sd expression", "up-regulated", "down-regulated", "Eisenberg 2013", "Hsiao 2001")
res <- as.data.frame(res,stringsAsFactors =F)
res$`Eisenberg 2013` <- as.factor(res$`Eisenberg 2013`)
res$`Hsiao 2001` <- as.factor(res$`Hsiao 2001`)
levels(res$`Eisenberg 2013`) <- c("no", "yes")
levels(res$`Hsiao 2001`) <- c("no", "yes")

write.table(res,file=paste0(namenorm, "/HK.txt"),sep = "\t",col.names = NA)


a <- res %>% filter(`mean expression`>=4) %>% filter(`sd expression`< 0.5) 
table(a$`Eisenberg 2013`)
table(res$`Eisenberg 2013`)
(table(a$`Eisenberg 2013`)[2]) / nrow(a)  

table(a$`Hsiao 2001`)
table(res$`Hsiao 2001`)

# PROVE PER CAPIRE PERCHE' LA DISTRIBUTIONE DEI GENI NEI NEUTROFILI E' DIVERSA
# tpm2<-log(tpmhk2+1)
# aac<-rep(1,127)
# aac[120:123]<-1.4
# TPM_DEC2 <- sweep(tpm2, MARGIN=2, STATS=aac,FUN="/")
# boxplot(TPM_DEC2, names=namesOrd, las=2)
# aac<-rep(0,127)
# aac[120:123]<-1
# TPM_DEC3 <- sweep(TPM_DEC2, MARGIN=2, STATS=aac,FUN="+")
# boxplot(TPM_DEC3, names=namesOrd, las=2)


#check HK normalized by adding median value to deconvolution normalization
# ac <- apply(tpmhk4,2, median)
# tpm2 <- log2(TPM_DEC+1)
# TPM_DEC2 <- sweep(tpm2, MARGIN=2, STATS=aac,FUN="+")
# tpmhk4b <- TPM_DEC2[genesHK, ]
# boxplot(tpmhk4b, names=namesOrd, las=2)
# boxplot(log2(TPM_HK+1), names=namesOrd, las=2)


#some stupid tests
# a<-c(1,2,0.5,0,0,0)
# b<-c(1,2,0.5,1,2,0.5)
# c<-c(4,8,2,4,5,3)
# boxplot(cbind(a,b,c))
# 
# a1 <- c(14,28,7,0,0,0)
# b1<- c(6.5, 13,3.25,6.5,13,6.5)
# c1<- c(7.2,15,10.8,5.4,7.2,9)
# 
# boxplot(cbind(a1,b1,c1))
# 
# apply(cbind(a1,b1,c1),2, median)
# 
# boxplot(cbind(a1/14,b1/6.5,c1/8.1))
