# make a backup of the files
cp *.fastq.gz /home/nis/giannim/work/RNAseqDEC_backup/

############### QUALITY CONTROL #######################
# Use FastQC for quality control
mkdir qc
/opt/fastqc/FastQC/fastqc -o qc -noextract *.fastq.gz
# I tried using HTQC, but it didn't work. I think it is better to leave it

######### TRIMMING AND FILTERING #############
# trimming
# for i in `ls *1.fastq.gz`
# do
# echo $i
# java -jar /home/nis/giannim/work/tools/Trimmomatic-0.36/trimmomatic-0.36.jar PE -threads 12 -trimlog ${i//_1.fastq.gz}_trim.log -phred33 -basein ${i} -baseout ${i//_1} ILLUMINACLIP:/home/nis/giannim/work/tools/Trimmomatic-0.36/adapters/NexteraPE-PE.fa:2:30:10 LEADING:3 TRAILING:3 SLIDINGWINDOW:4:15 MINLEN:36 >> trimmomatic_log_all.txt 2>&1
# done
# Although doing the trimming, it removes a lot of low quality reads from the second end paired data, it also removes the corresponding high quality reads from the first paired end. The low quality reads will not properly aligned by STAR successively anyway. Therefore I decided not to use the trimmed data.

############### ADDING ERCC skipe-ins #########################
# before you start any analysis, remember that you can add the spike-in sequences and their annotation on the genome, transcriptome and annotation file you are going to use for the alignment. Search for ERCC and follow the instruction in the READE file.


###################### ALIGNMENT and COUNTING OF FEATURES ####################
################################################################################
##### GENOME INDEXING
# After having checked the HTML report produced by FASTQC, I saw that no files were of bad quality. Therefore I don't need to do any trimming or rRNA depletion. 
#First of all, I retrieve the Genome build in FASTA format and the annotation file in GTF format from GENCODE
mkdir GenomeDir   # if you have already a directory for your genomes, delete this step
#downolad the genome build
wget -P GenomeDir ftp://ftp.sanger.ac.uk/pub/gencode/Gencode_human/release_26/GRCh38.primary_assembly.genome.fa.gz #remove -P GenomeDir if you want to download it in the working directory
# download the annotation in the annotation directory
wget ftp://ftp.sanger.ac.uk/pub/gencode/Gencode_human/release_26/gencode.v26.annotation.gtf.gz
# Add the spike-ins with something like: cat ERCC92.fa >> GRCh38.primary_assembly.genome.fa    and     cat ERCC92.gtf >> gencode.v26.annotation.gtf
# Here I create the reference genome with STAR
mkdir Index_STAR_sj49
/home/nis/giannim/work/tools/STAR-2.5.3a/bin/Linux_x86_64/STAR --runThreadN 12 --runMode genomeGenerate --genomeDir /home/nis/giannim/work/Genomes/human_ERCC/Index_STAR_sj49 --genomeFastaFiles /home/nis/giannim/work/Genomes/human_ERCC/GRCh38.primary_assembly.genome.ercc.fa --sjdbGTFfile /home/nis/giannim/work/annotations/human_ERCC/gencode.v26.annotation.ercc.gtf --sjdbOverhang 49

##### GENOME ALIGNMENT
# check the files:
set -- `ls *fastq.gz`
echo ${@:1:4}
# let's run STAR now for the mapping
cd /home/nis/giannim/work/RNA_Seq_dec
for i in `ls -1 *.fastq.gz | cut -d '_' -f1 | sort -u`
do
	echo $i
	/home/nis/giannim/work/tools/STAR-2.5.3a/bin/Linux_x86_64/STAR --runThreadN 12 --genomeDir /home/nis/giannim/work/Genomes/human_ERCC/Index_STAR_sj49 --readFilesIn ${i}_1.fastq.gz ${i}_2.fastq.gz --readFilesCommand gunzip -c --outFileNamePrefix ${i}_ --seedSearchStartLmax 30
done
rm -r *STARtmp

# using the index genome created with --sjdbOverhang 49 and mapping with --seedSearchStartLmax 30 (see https://groups.google.com/forum/#!topic/rna-star/h9oh10UlvhI), I got more genes uniquely mapped. I think that it was due to an increase of sensitivity. Therefore I used these parameters.

###### SORTING AND INDEXING WITH SAMTOOLS
# let's run samtools to sort and index the the SAM files generated by STAR. You can delete the SAM files after this since they are large.
for i in RHH*.sam
do
	sample=${i/\_Aligned\.out\.sam}
	echo $sample
	/home/nis/giannim/work/tools/samtools-1.3.1/samtools view -b ${sample}_Aligned.out.sam | /home/nis/giannim/work/tools/samtools-1.3.1/samtools sort -@ 6 -O bam -T tmp - > ${sample}.bam
	/home/nis/giannim/work/tools/samtools-1.3.1/samtools index ${sample}.bam
done

######COUNTING THE FEATURES
# now run featureCounts. Fist you can use only the parameter --largestOverlap
/home/nis/giannim/work/tools/subread-1.5.2-source/bin/featureCounts -T 6 -p -t exon -g gene_id --largestOverlap -a /home/nis/giannim/work/annotations/human_ERCC/gencode.v26.annotation.ercc.gtf -o counts_largeOv.txt *.bam >> counts_largeOv_log.txt 2>&1
#in case you want to count also reads that overlap to multiple regions you can use:
# /home/nis/giannim/work/tools/subread-1.5.1-source/bin/featureCounts -T 6 -p -t exon -g gene_id --largestOverlap -M --fraction -a /home/nis/giannim/work/Genomes/human_ERCC/gencode.v25_ERCC.gtf -o counts_largeOv_multi.txt *.bam >> counts_largeOv_log_multi.txt 2>&1
#you could also use HTseq but you have to use SAM files for that:
# to convert the bam files to sam files (if you previously deleted them) you can use: 
# for i in RHH*.bam
# do
	# sample=${i/\.bam}
	# echo $sample
	# htseq-count -f bam -m union -s no ${i} /home/nis/giannim/work/Genomes/human/gencode.v25.annotation.gtf > ${sample}_htseq.txt
# done

##################### PSEUDOALIGNMENT TO TRANSCRIPTOME #########################
################################################################################
####### DOWNLOAD TRANSCRIPTOME
# I first of all download the transcriptome from gencode:
wget ftp://ftp.sanger.ac.uk/pub/gencode/Gencode_human/release_26/gencode.v26.transcripts.fa.gz
# or this from Ensembl: wget ftp://ftp.ensembl.org/pub/release-87/fasta/homo_sapiens/cdna/Homo_sapiens.GRCh38.cdna.all.fa.gz
# Add the spike-ins with something like: cat ERCC92.fa >> gencode.v26.transcripts.fa 
####### INDEXING OF THE TRANSCRIPTOME
kallisto index -i gencode26_ercc.idx gencode.v26.transcripts.ercc.fa
####### QUANTIFICATION
# remember that you can only run one pair at a time!
for i in `ls -1 *.fastq.gz | cut -d '_' -f1 | sort -u`
do
	echo $i
	kallisto quant -i /home/nis/giannim/work/Transcriptomes/human_ERCC/gencode26_ercc.idx -o ${i}_kallisto -b 100 -t12 ${i}_1.fastq.gz ${i}_2.fastq.gz >> kallisto_log_all.txt 2>&1
done

########### QUALITY ASSESSMENT LOG FILES ############
# to do a quality assessment of all the previous analysis, I ran multiqc on all the log files.
# Open the command prompt, go to the directory that contains the log files and run:
python C:\Users\GianniMonaco\Desktop\softwares\MultiQC-master\scripts\multiqc .
python C:\Users\GianniMonaco\Desktop\softwares\MultiQC-master\scripts\multiqc -m trimmomatic . # you can use this one if you only want to run trimmomatic
# from the server use:
source /opt/multiqc/MultiQC-master/env/bin/activate
multiqc .

########### Normalization and differential expression ##########
# Now continue using R scripts to process and analyse the expression data


